c ###########################################################
c
      Program maketab
      Implicit NONE
c
c A program to construct the look-up tables of Mie coefficients
c for use by microwave radiometer and radar simulations.
c 
c "mxpts_temp" and "mxpts_hydro" are the total numbers of temperature
c and hydrometeor water content, respectively, specified in the LUTs.
c Any changes to these numbers must be properly reflected in
c "src/simulator/MAINparameters.inc" for use by the simulator.
c
      Integer mxpts_temp,mxpts_hydro
      PARAMETER ( mxpts_temp = 12, mxpts_hydro = 50 )
c
c "mxfreqMW" and "mxfreqRD" are the numbers of frequency channels
c for microwave radiometer and radar, respectively.
c
      integer mxfreqMW,mxfreqRD
      Parameter ( mxfreqMW = 5, mxfreqRD = 1 )
c
      CHARACTER*13 fname_out(0:6)
      character*3 fnametag_rdmtr(mxfreqMW)
      character*3 fnametag_radar(mxfreqRD)
      character*(*) dir_out
      character*100 pathandfile
      Logical rdmtr,radar
C
      REAL pts_hydro(mxpts_hydro)
      REAL freq_rdmtr(mxfreqMW), freq_radar(mxfreqRD)
      Real pts_temp_w(mxpts_temp), pts_temp_i(mxpts_temp)
      REAL kext(mxpts_temp, mxpts_hydro) 
      REAL salb(mxpts_temp, mxpts_hydro)
      REAL asym(mxpts_temp, mxpts_hydro)
      REAL pbck(mxpts_temp, mxpts_hydro)
      REAL kext_m(mxpts_hydro), salb_m(mxpts_hydro)
      REAL asym_m(mxpts_hydro), pbck_m(mxpts_hydro)
      Real frequency , bscaR
      Integer nspec,i,j,nfreq,k
      Integer npts_temp,npts_hydro
C
C Directory for the output lookup table.
C
      Parameter (dir_out = '../../Mie-table/')
c
c LUT values of hydrometeor water content (in [g/m3])
c
      data pts_hydro/ 0.001, 0.002, 0.003, 0.004, 0.005, 0.006,
     $             0.007, 0.008, 0.009, 0.01, 0.02, 0.03, 0.04,
     $             0.05, 0.06, 0.07, 0.08, 0.09, 0.1, 0.2, 0.3,
     $             0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 2.0, 3.0,
     $             4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 13*0 /
c
c LUT values of temperature (in [K]) for liquid hydrometeors,
c i.e., rain and cloud water.
c
      data pts_temp_w/ 255., 260., 265., 270., 275., 280., 285., 
     $                 290., 295., 300., 305., 310. /
c
c LUT values of temperature (in [K]) for frozen hydrometeors,
c i.e., snow, grapel, hail, and cloud ice.
c
      data pts_temp_i/ 180., 190., 200., 210., 220., 230., 240., 
     $                 250., 260., 270., 275., 280. /
c
c MW radiometer channel frequencies (in [GHz])
c
      data freq_rdmtr / 10.7, 19.35, 21.3, 37.0, 85.5 /
c  Labels for the file names of the output lookup table
c    # The labels should be three characters. Make sure that
c    # "nch(i)" in "src/simulator/microw/open-lut-mwr.f" is
c    # consistently defined with the labels given here.
c
      data fnametag_rdmtr / '10G','19G','21G','37G','85G' /
c
c Radar channel frequencies (in [GHz])
c
      data freq_radar / 13.8 /
c  Labels for the file names of the output lookup table
c    # The labels should be three characters. Make sure that
c    # "nch(i)" in "src/simulator/radar/open-lut-rdr.f" is
c    # consistently defined with the labels given here.
c
      data fnametag_radar / '14G' /
c
c Mie LUTs are constructed for MW radiometer simulation when rdmtr=.true.
c
      rdmtr = .true.
c
c Mie LUTs are constructed for radar simulation when radar=.true.
c
      radar = .true. 
C
c ### Ouput kext, salb, and asym for Radiometer ###
c
      If ( rdmtr ) then
c
      npts_hydro = 37
      npts_temp  = 12
c
      write(*,*) '- Microwave Radiometer- '
c
      do 2000 nfreq = 1 , mxfreqMW
c
      write(*,*) 'Freq = ',freq_rdmtr(nfreq)
      fname_out(0) = 'ML-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      fname_out(1) = 'CI-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      fname_out(2) = 'CW-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      fname_out(3) = 'GR-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      fname_out(4) = 'HL-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      fname_out(5) = 'SN-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      fname_out(6) = 'RR-mwr'//fnametag_rdmtr(nfreq)//'.dat' 
      frequency = freq_rdmtr(nfreq)
c
c Melting particles
c
      nspec = 0
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Melting particles -'
c 
c       Write header for output file
c
      write(2,*) npts_hydro
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency
 90   Format ( 1P3E14.6 )
 91   Format ( 1F12.5 )
 92   Format ( 1P50E13.5 )
 95   Format ( 2I5 )
c
c       Initialize scattering parameters
c
      do 100  j = 1, mxpts_hydro
           kext_m(j) = 0.
           salb_m(j) = 0.
           pbck_m(j) = 0.
 100    continue 
c
        do 200  j  = 1, npts_hydro
            Call df_mie_melt(frequency,273.16,
     &                .5*pts_hydro(j),.5*pts_hydro(j),
     &                kext_m(j),asym_m(j),salb_m(j),pbck_m(j))
           write(2,90) KEXT_m(j), SALB_m(j), ASYM_m(j)
 200    Continue
        close(2)
c
c cloud ice
c
      nspec = 1
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Cloud ice -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 101  i = 1, mxpts_temp
         do 101  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  101   continue 
c
c       Calculate scattering parameters
c 
        do 201  i = 1, npts_temp
         do 201  j  = 1, npts_hydro
c
            call mie_ci_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), ASYM(i,j)
  201   continue
        close (2)
c
c cloud water
c
      nspec = 2
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Cloud water -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 102  i = 1, mxpts_temp
         do 102  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  102   continue 
c
c       Calculate scattering parameters
c 
        do 202  i = 1, npts_temp
         do 202  j  = 1, npts_hydro
c
            call mie_clw_bs(frequency, pts_temp_w(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), ASYM(i,j)
  202   continue
        close (2)
c
c graupel
c
      nspec = 3
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Graupel -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 103  i = 1, mxpts_temp
         do 103  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  103   continue 
c
c       Calculate scattering parameters
c 
        do 203  i = 1, npts_temp
         do 203  j  = 1, npts_hydro
c
            call mie_graup_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j), 
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), ASYM(i,j)
  203   continue
        close (2)
c
c
c hail
c
      nspec = 4
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Hail -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 104  i = 1, mxpts_temp
         do 104  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  104   continue 
c
c       Calculate scattering parameters
c 
        do 204  i = 1, npts_temp
         do 204  j  = 1, npts_hydro
c
            call mie_hail_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), ASYM(i,j)
  204   continue
        close (2)
c
c snow
c
      nspec = 5
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Snow -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 105  i = 1, mxpts_temp
         do 105  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  105   continue 
c
c       Calculate scattering parameters
c 
        do 205  i = 1, npts_temp
         do 205  j  = 1, npts_hydro
c
            call mie_snow_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j), 
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), ASYM(i,j)
 205    continue
        close (2)
c
c
c Rain water
c
      nspec = 6
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Rain -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 106  i = 1, mxpts_temp
         do 106  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  106   continue 
c
c       Calculate scattering parameters
c 
        do 206  i = 1, npts_temp
         do 206  j  = 1, npts_hydro
c
            call mie_rain_bs(frequency, pts_temp_w(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j), bscaR)
c
           write(2,90) KEXT(i,j), SALB(i,j), ASYM(i,j)
 206    continue
        close (2)
c
 2000 continue
C
      Endif
C
C
c ### Ouput kext and pbck for Radar ###
c
      If ( radar ) then
c
      npts_hydro = 37
      npts_temp  = 12
c
      write(*,*) '- Radar - '
c
      do 4000 nfreq = 1 , mxfreqRD
c
      write(*,*) 'Freq = ',freq_radar(nfreq)
      fname_out(0) = 'ML-rdr'//fnametag_radar(nfreq)//'.dat' 
      fname_out(1) = 'CI-rdr'//fnametag_radar(nfreq)//'.dat' 
      fname_out(2) = 'CW-rdr'//fnametag_radar(nfreq)//'.dat' 
      fname_out(3) = 'GR-rdr'//fnametag_radar(nfreq)//'.dat' 
      fname_out(4) = 'HL-rdr'//fnametag_radar(nfreq)//'.dat' 
      fname_out(5) = 'SN-rdr'//fnametag_radar(nfreq)//'.dat' 
      fname_out(6) = 'RR-rdr'//fnametag_radar(nfreq)//'.dat' 
c
      frequency = freq_radar(nfreq)
c
c Melting particles
c
      nspec = 0
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Melting particles -'
c 
c
c       Write header for output file
c
      write(2,*) npts_hydro
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency
c
c       Initialize scattering parameters
c
      do 300  j = 1, mxpts_hydro
           kext_m(j) = 0.
           salb_m(j) = 0.
           pbck_m(j) = 0.
 300  continue 
c
         do 400  j  = 1, npts_hydro
            Call df_mie_melt(frequency,273.16,
     &                .5*pts_hydro(j),.5*pts_hydro(j),
     &                kext_m(j),asym_m(j),salb_m(j),pbck_m(j))
           write(2,90) KEXT_m(j), SALB_m(j), PBCK_m(j)
 400    Continue
        close(2)
c
c cloud ice
c
      nspec = 1
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Cloud ice -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 301  i = 1, mxpts_temp
         do 301  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  301   continue 
c
c       Calculate scattering parameters
c 
        do 401  i = 1, npts_temp
         do 401  j  = 1, npts_hydro
c
            call mie_ci_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), PBCK(i,j)
  401   continue
        close (2)
c
c cloud water
c
      nspec = 2
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Cloud water -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 302  i = 1, mxpts_temp
         do 302  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  302   continue 
c
c       Calculate scattering parameters
c 
        do 402  i = 1, npts_temp
         do 402  j  = 1, npts_hydro
c
            call mie_clw_bs(frequency, pts_temp_w(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), PBCK(i,j)
  402   continue
        close (2)
c
c graupel
c
      nspec = 3
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Graupel -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 303  i = 1, mxpts_temp
         do 303  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  303   continue 
c
c       Calculate scattering parameters
c 
        do 403  i = 1, npts_temp
         do 403  j  = 1, npts_hydro
c
            call mie_graup_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j), 
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), PBCK(i,j)
  403   continue
        close (2)
c
c
c hail
c
      nspec = 4
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Hail -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 304  i = 1, mxpts_temp
         do 304  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  304   continue 
c
c       Calculate scattering parameters
c 
        do 404  i = 1, npts_temp
         do 404  j  = 1, npts_hydro
c
            call mie_hail_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), PBCK(i,j)
  404   continue
        close (2)
c
c snow
c
      nspec = 5
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,Status='NEW')
      write(*,*) 'Snow -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_i(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 305  i = 1, mxpts_temp
         do 305  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  305   continue 
c
c       Calculate scattering parameters
c 
        do 405  i = 1, npts_temp
         do 405  j  = 1, npts_hydro
c
            call mie_snow_bs(frequency, pts_temp_i(i), 
     $                   pts_hydro(j), 
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j))
c
           write(2,90) KEXT(i,j), SALB(i,j), PBCK(i,j)
 405    continue
        close (2)
c
c
c Rain water
c
      nspec = 6
      pathandfile = dir_out//fname_out(nspec)
      open(2,file=pathandfile,STATUS='NEW')
      write(*,*) 'Rain -'
c
c       Write header for output file
c
      write(2,95) npts_temp, npts_hydro
      write(2,92) (pts_temp_w(i),i=1,npts_temp)
      write(2,92) (pts_hydro(i),i=1,npts_hydro)
      write(2,91) frequency

        do 306  i = 1, mxpts_temp
         do 306  j = 1, mxpts_hydro
           kext(i,j) = 0.
           salb(i,j) = 0.
           asym(i,j) = 0.
  306   continue 
c
c       Calculate scattering parameters
c 
        do 406  i = 1, npts_temp
         do 406  j  = 1, npts_hydro
c
            call mie_rain_bs(frequency, pts_temp_w(i),
     $                   pts_hydro(j),
     $                   kext(i,j), salb(i,j), 
     $                   asym(i,j), pbck(i,j), bscaR)
c
           write(2,90) KEXT(i,j), SALB(i,j), PBCK(i,j)
 406    continue
        close (2)
 4000 Continue
C
      Endif
C
      stop
      end
c
c
