      SUBROUTINE ML_97 (W_MM, F_W, F_A, F_I, F_AA, F_II,
     +                  DENSE_I, DENSE_S, EPS_W, EPS_A, EPS_I, MEFF)
**
** model for effective permittivity of air-ice-water mixture
** following Meneghini and Liao (submitted to IEEE TGARS 1997)
** valid for wavelengths between 3 and 30 mm, i.e. frequencies
** between 10 and 100 GHz 
**
      IMPLICIT NONE

      SAVE

      REAL W_MM, F_W, F_A, F_I, F_AA, F_II
      REAL DENSE_S, DENSE_I, AR, BR, AI, BI
      REAL CAR (0:3), CBR (0:3), CAI (0:3), CBI (0:3)
      REAL REQUI (2)

      COMPLEX EPS_W, EPS_A, EPS_I, EPS_S, MEFF
      COMPLEX EPS_WW, EPS_AA, EPS_II
      COMPLEX E_WI, E_AI, K_I, CEQUI

      INTEGER I100

      EQUIVALENCE (CEQUI, REQUI (1))

** coefficients for parameterization of mean electric field of 
** water-ice mixture
      DATA CAR / -6.97    E-04,  6.775 E-04, -4.074 E-05,  7.148 E-07 /
      DATA CBR /  0.76635 E+00, -0.099 E+00,  4.90  E-03, -7.944 E-05 /
      DATA CAI / -2.113   E-03,  2.223 E-04, -6.072 E-06, 4.6025 E-08 /
      DATA CBI /  0.4116  E+00, -0.193 E-01,  2.365 E-04, 1.7503 E-06 /

** conversion of input permittivities to negative imaginary parts
      CEQUI  = EPS_W
      EPS_WW = CMPLX (REQUI (1), -1.0 * REQUI (2))
      CEQUI  = EPS_I
      EPS_II = CMPLX (REQUI (1), -1.0 * REQUI (2))
      CEQUI  = EPS_A
      EPS_AA = CMPLX (REQUI (1), -1.0 * REQUI (2))

** permittivity of snow
      K_I   = (EPS_II - 1.0) / (EPS_II + 2.0)
      EPS_S = (DENSE_I + 2.0 * K_I * DENSE_S)
     +      / (DENSE_I -       K_I * DENSE_S)

** parameterization of mean electric field of water-ice mixture
      AR = 0.0
      BR = 0.0
      AI = 0.0
      BI = 0.0

      DO 100 I100 = 0, 3
         AR = AR + CAR (I100) * (W_MM ** I100) 
         BR = BR + CBR (I100) * (W_MM ** I100) 
         AI = AI + CAI (I100) * (W_MM ** I100) 
         BI = BI + CBI (I100) * (W_MM ** I100) 
 100  CONTINUE

** mean electric field ratios of 2-component mixtures
      E_WI = CMPLX (AR * F_W * 100.0 + BR, AI * F_W * 100.0 + BI)

      IF (F_AA .GT. 0.0) THEN
         E_AI =  F_II * (EPS_S  - EPS_II) 
     +        / (F_AA * (EPS_AA - EPS_S ))
      ELSE
         E_AI = CMPLX (0.0,0.0)
      END IF

** effective permittivity of 3-component mixture
      CEQUI = (EPS_WW * F_W * E_WI + EPS_AA * F_A * E_AI + EPS_II * F_I)
     +     / (          F_W * E_WI +          F_A * E_AI +          F_I)

      MEFF  = CSQRT (CMPLX (REQUI (1), ABS (REQUI (2))))

      RETURN
      END
