      Subroutine SIMradar ( lut, freq_radar, umu, press_lev, temp_lev, 
     &                      wind_lev, hgt_lev, relhum_avg, 
     &                      avg_pressure, avg_H2Opressure, 
     &                      cloud_water_avg, rain_avg, cloud_ice_avg, 
     &                      snow_avg, graupel_avg, hail_avg, ics,
     &                      ngridx, ngridy, nlyr, deltax, deltay,
     &                      dBZ_out, PIA_out )
      Implicit NONE
c
c This subroutine, called by "../RTSIM.f", is the core routine for
c radar echo simulation. 
c
c Inputs:
c  - Running option
c     lut
c  - Sensor parameters
c     freq_radar, umu
c  - Geophysical parameters
c     press_lev, temp_lev, wind_lev (not in use), hgt_lev, 
c     relhum_avg, avg_pressure, avg_H2Opressure, cloud_water_avg, 
c     rain_avg, cloud_ice_avg, snow_avg, graupel_avg, hail_avg, ics
c  - Model spatial dimensions
c     ngridx, ngridy, nlyr, deltax, deltay
c
c Outputs:
c     dBZ_out, PIA_out
c
      Include '../MAINparameters.inc'
c
c ############ Mie LUT ##############
c
      real *4 kextrr_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar) 
      real *4 salbrr_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 pbckrr_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 kextci_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar) 
      real *4 salbci_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 pbckci_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 kextcw_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 salbcw_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 pbckcw_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 kexthl_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar) 
      real *4 salbhl_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 pbckhl_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 kextsn_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 salbsn_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 pbcksn_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 kextgr_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar) 
      real *4 salbgr_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 pbckgr_lut_nf(mxpts_temp, mxpts_hydro, mxfreq_radar)
      real *4 kextml_lut_nf(mxpts_hydro, mxfreq_radar) 
      real *4 salbml_lut_nf(mxpts_hydro, mxfreq_radar)
      real *4 pbckml_lut_nf(mxpts_hydro, mxfreq_radar)
      real *4 kextci_lut(mxpts_temp, mxpts_hydro) 
      real *4 salbci_lut(mxpts_temp, mxpts_hydro)
      real *4 pbckci_lut(mxpts_temp, mxpts_hydro)
      real *4 kextcw_lut(mxpts_temp, mxpts_hydro)
      real *4 salbcw_lut(mxpts_temp, mxpts_hydro)
      real *4 pbckcw_lut(mxpts_temp, mxpts_hydro)
      real *4 kexthl_lut(mxpts_temp, mxpts_hydro) 
      real *4 salbhl_lut(mxpts_temp, mxpts_hydro)
      real *4 pbckhl_lut(mxpts_temp, mxpts_hydro)
      real *4 kextrr_lut(mxpts_temp, mxpts_hydro) 
      real *4 salbrr_lut(mxpts_temp, mxpts_hydro)
      real *4 pbckrr_lut(mxpts_temp, mxpts_hydro)
      real *4 kextsn_lut(mxpts_temp, mxpts_hydro)
      real *4 salbsn_lut(mxpts_temp, mxpts_hydro)
      real *4 pbcksn_lut(mxpts_temp, mxpts_hydro)
      real *4 kextgr_lut(mxpts_temp, mxpts_hydro) 
      real *4 salbgr_lut(mxpts_temp, mxpts_hydro)
      real *4 pbckgr_lut(mxpts_temp, mxpts_hydro)
      real *4 kextml_lut(mxpts_hydro) 
      real *4 salbml_lut(mxpts_hydro)
      real *4 pbckml_lut(mxpts_hydro)
      Integer npts_temp,npts_hydro
c
c Radiative quantities
c
      ! Internal variables
      real *4 tavg, tavg_c, Pres
      real *4 atm_ext(mxgridx,mxgridy,mxlyr)
      real *4 kextcw(mxgridx,mxgridy,mxlyr)
      real *4 salbcw(mxgridx,mxgridy,mxlyr)
      real *4 asymcw(mxgridx,mxgridy,mxlyr)
      real *4 kextrr(mxgridx,mxgridy,mxlyr)
      real *4 salbrr(mxgridx,mxgridy,mxlyr)
      real *4 asymrr(mxgridx,mxgridy,mxlyr)
      real *4 kextci(mxgridx,mxgridy,mxlyr)
      real *4 salbci(mxgridx,mxgridy,mxlyr)
      real *4 asymci(mxgridx,mxgridy,mxlyr)
      real *4 kextsn(mxgridx,mxgridy,mxlyr)
      real *4 salbsn(mxgridx,mxgridy,mxlyr) 
      real *4 asymsn(mxgridx,mxgridy,mxlyr)
      real *4 kextgr(mxgridx,mxgridy,mxlyr)
      real *4 salbgr(mxgridx,mxgridy,mxlyr)
      real *4 asymgr(mxgridx,mxgridy,mxlyr)
      real *4 kexthl(mxgridx,mxgridy,mxlyr)
      real *4 salbhl(mxgridx,mxgridy,mxlyr)
      real *4 asymhl(mxgridx,mxgridy,mxlyr)
      real *4 kextml(mxgridx,mxgridy,mxlyr)
      real *4 salbml(mxgridx,mxgridy,mxlyr)
      real *4 asymml(mxgridx,mxgridy,mxlyr)
      real *4 kexttot(mxgridx,mxgridy,mxlyr)
      real *4 salbtot(mxgridx,mxgridy,mxlyr)
      real *4 asymtot(mxgridx,mxgridy,mxlyr)
      real *4 pbckcw(mxgridx,mxgridy,mxlyr)
      real *4 pbckrr(mxgridx,mxgridy,mxlyr)
      real *4 pbckci(mxgridx,mxgridy,mxlyr)
      real *4 pbcksn(mxgridx,mxgridy,mxlyr)
      real *4 pbckgr(mxgridx,mxgridy,mxlyr)
      real *4 pbckhl(mxgridx,mxgridy,mxlyr)
      real *4 pbckml(mxgridx,mxgridy,mxlyr)
      real *4 sbacktot(mxgridx,mxgridy,mxlyr)
      real *4 kextml0,salbml0,asymml0,pbckml0
c
      real *4 dBZ(mxgridx,mxgridy,mxlyr)
      real *4 PIA(mxgridx,mxgridy)
c
      real *4 freqcy
      real *4 pi 
c
      real *4 kext_hyd,f_frz
      integer nzfrz(mxgridx,mxgridy)
c
      character*90 out_profile_name
      Logical intplut
c
c Simplified treatment of melting particles
c
      Parameter ( pi = 3.1415926536 )
      ! internal loop counters
      integer    i, j, k, l, m, nx, ny, nz
      integer    ix, iy, nf
c
      Character*100 pathandfile
      Real *4 bscaR
c
c Read Mie lookup table.
c
      If ( lut ) Call open_lut_rdr(
     $                    kextci_lut_nf,salbci_lut_nf,pbckci_lut_nf,
     &                    kextcw_lut_nf,salbcw_lut_nf,pbckcw_lut_nf,
     &                    kextrr_lut_nf,salbrr_lut_nf,pbckrr_lut_nf,
     &                    kextsn_lut_nf,salbsn_lut_nf,pbcksn_lut_nf,
     &                    kextgr_lut_nf,salbgr_lut_nf,pbckgr_lut_nf,
     &                    kexthl_lut_nf,salbhl_lut_nf,pbckhl_lut_nf,
     &                    kextml_lut_nf,salbml_lut_nf,pbckml_lut_nf,
     &                    pts_temp_w, pts_temp_i, pts_hydro,
     &                    npts_temp, npts_hydro )
c
c Identify the freezing level
c
      do nx = 1,ngridx
         do ny = 1,ngridy
            do nz = 1,nlyr
               If (temp_lev(nx,ny,nz-1).ge.273.16 .and.
     &             temp_lev(nx,ny,nz)  .le.273.16) then
                  nzfrz(nx,ny) = nz
                  Goto 23
               Endif
            end do
 23      end do
      end do
c
c     Loop over the channel frequencies
c
      do 200 nf = 1, mxfreq_radar
c
        write(*,*) 'Radar simulation --------'
        freqcy = freq_radar(nf)
        write(*,*) ' nfreq(radar): ',nf,' at',freqcy,' GHz'
c
        Do i = 1 , mxpts_temp
         Do j = 1 , mxpts_hydro
            kextrr_lut(i,j) = kextrr_lut_nf(i,j,nf)
            kextci_lut(i,j) = kextci_lut_nf(i,j,nf)
            kextcw_lut(i,j) = kextcw_lut_nf(i,j,nf)
            kextsn_lut(i,j) = kextsn_lut_nf(i,j,nf)
            kextgr_lut(i,j) = kextgr_lut_nf(i,j,nf)
            kexthl_lut(i,j) = kexthl_lut_nf(i,j,nf)
            salbrr_lut(i,j) = salbrr_lut_nf(i,j,nf)
            salbci_lut(i,j) = salbci_lut_nf(i,j,nf)
            salbcw_lut(i,j) = salbcw_lut_nf(i,j,nf)
            salbsn_lut(i,j) = salbsn_lut_nf(i,j,nf)
            salbgr_lut(i,j) = salbgr_lut_nf(i,j,nf)
            salbhl_lut(i,j) = salbhl_lut_nf(i,j,nf)
            pbckrr_lut(i,j) = pbckrr_lut_nf(i,j,nf)
            pbckci_lut(i,j) = pbckci_lut_nf(i,j,nf)
            pbckcw_lut(i,j) = pbckcw_lut_nf(i,j,nf)
            pbcksn_lut(i,j) = pbcksn_lut_nf(i,j,nf)
            pbckgr_lut(i,j) = pbckgr_lut_nf(i,j,nf)
            pbckhl_lut(i,j) = pbckhl_lut_nf(i,j,nf)
         End do
        End do
        Do j = 1 , mxpts_hydro
            kextml_lut(j) = kextml_lut_nf(j,nf)
            salbml_lut(j) = salbml_lut_nf(j,nf)
            pbckml_lut(j) = pbckml_lut_nf(j,nf)
        End do
c
      do 210 nx = 1, ngridx
        do 210 ny = 1, ngridy
          f_frz = (temp_lev(nx,ny,nzfrz(nx,ny)-1) - 273.16) /
     &            (temp_lev(nx,ny,nzfrz(nx,ny)-1) - 
     &             temp_lev(nx,ny,nzfrz(nx,ny)))
c
          do 220 nz = 1, nlyr
c          
            Tavg = 0.5*(temp_lev(nx,ny,nz-1)+temp_lev(nx,ny,nz))
            Tavg_c = Tavg - 273.16
            Pres = 
     +          (press_lev(nx,ny,nz) - press_lev(nx,ny,nz-1) )/
     +           log(press_lev(nx,ny,nz)/press_lev(nx,ny,nz-1))
C           
c Gas absorption code updated (11/06/03), provided by Chris.
c           
            call absorb(freqcy, Tavg, Pres, 
     +                  relhum_avg(nx,ny,nz), 0., atm_ext(nx,ny,nz))
c
c initialization
c
            kextrr(nx,ny,nz) = 0.
            salbrr(nx,ny,nz) = 0.
            pbckrr(nx,ny,nz) = 0.
            
            kextci(nx,ny,nz) = 0.
            salbci(nx,ny,nz) = 0.
            pbckci(nx,ny,nz) = 0.

            kextcw(nx,ny,nz) = 0.
            salbcw(nx,ny,nz) = 0.
            pbckcw(nx,ny,nz) = 0.

            kextsn(nx,ny,nz) = 0.
            salbsn(nx,ny,nz) = 0.
            pbcksn(nx,ny,nz) = 0.

            kextgr(nx,ny,nz) = 0.
            salbgr(nx,ny,nz) = 0.
            pbckgr(nx,ny,nz) = 0.

            kexthl(nx,ny,nz) = 0.
            salbhl(nx,ny,nz) = 0.
            pbckhl(nx,ny,nz) = 0.
c
            If ( lut ) then
c
c Read LUT
c
            Call mie_lut (Tavg,rain_avg(nx,ny,nz),
     &                    kextrr_lut,salbrr_lut,pbckrr_lut,
     &                    pts_temp_w, pts_hydro,
     &                    npts_temp, npts_hydro,
     &                    mxpts_temp, mxpts_hydro,
     &                    kextrr(nx,ny,nz),salbrr(nx,ny,nz),
     &                    pbckrr(nx,ny,nz))
c
            Call mie_lut (Tavg,cloud_ice_avg(nx,ny,nz),
     &                    kextci_lut,salbci_lut,pbckci_lut,
     &                    pts_temp_i, pts_hydro,
     &                    npts_temp, npts_hydro,
     &                    mxpts_temp, mxpts_hydro,
     &                    kextci(nx,ny,nz),salbci(nx,ny,nz),
     &                    pbckci(nx,ny,nz))
c
            Call mie_lut (Tavg,cloud_water_avg(nx,ny,nz),
     &                    kextcw_lut,salbcw_lut,pbckcw_lut,
     &                    pts_temp_w, pts_hydro,
     &                    npts_temp, npts_hydro,
     &                    mxpts_temp, mxpts_hydro,
     &                    kextcw(nx,ny,nz),salbcw(nx,ny,nz),
     &                    pbckcw(nx,ny,nz))
c
            Call mie_lut (Tavg,snow_avg(nx,ny,nz),
     &                    kextsn_lut,salbsn_lut,pbcksn_lut,
     &                    pts_temp_i, pts_hydro,
     &                    npts_temp, npts_hydro,
     &                    mxpts_temp, mxpts_hydro,
     &                    kextsn(nx,ny,nz),salbsn(nx,ny,nz),
     &                    pbcksn(nx,ny,nz))
c
            Call mie_lut (Tavg,graupel_avg(nx,ny,nz),
     &                    kextgr_lut,salbgr_lut,pbckgr_lut,
     &                    pts_temp_i, pts_hydro,
     &                    npts_temp, npts_hydro,
     &                    mxpts_temp, mxpts_hydro,
     &                    kextgr(nx,ny,nz),salbgr(nx,ny,nz),
     &                    pbckgr(nx,ny,nz))
c
            Call mie_lut (Tavg,hail_avg(nx,ny,nz),
     &                    kexthl_lut,salbhl_lut,pbckhl_lut,
     &                    pts_temp_i, pts_hydro,
     &                    npts_temp, npts_hydro,
     &                    mxpts_temp, mxpts_hydro,
     &                    kexthl(nx,ny,nz),salbhl(nx,ny,nz),
     &                    pbckhl(nx,ny,nz))
c
            Else 
c
            call mie_clw_bs(freqcy, Tavg, cloud_water_avg(nx,ny,nz), 
     $                  kextcw(nx,ny,nz), salbcw(nx,ny,nz), 
     $                  asymcw(nx,ny,nz), pbckcw(nx,ny,nz))
c
            call mie_rain_bs(freqcy, Tavg, rain_avg(nx,ny,nz),
     $                   kextrr(nx,ny,nz), salbrr(nx,ny,nz), 
     $                   asymrr(nx,ny,nz), pbckrr(nx,ny,nz), bscaR)
c
            call mie_ci_bs( freqcy, Tavg, cloud_ice_avg(nx,ny,nz),
     $                   kextci(nx,ny,nz), salbci(nx,ny,nz), 
     $                   asymci(nx,ny,nz), pbckci(nx,ny,nz))
c
            call mie_snow_bs( freqcy, Tavg, snow_avg(nx,ny,nz), 
     $                   kextsn(nx,ny,nz), salbsn(nx,ny,nz), 
     $                   asymsn(nx,ny,nz), pbcksn(nx,ny,nz))
c
            call mie_graup_bs( freqcy, Tavg, graupel_avg(nx,ny,nz), 
     $                   kextgr(nx,ny,nz), salbgr(nx,ny,nz), 
     $                   asymgr(nx,ny,nz), pbckgr(nx,ny,nz))
c
            call mie_hail_bs( freqcy, Tavg, hail_avg(nx,ny,nz),
     $                   kexthl(nx,ny,nz), salbhl(nx,ny,nz), 
     $                   asymhl(nx,ny,nz), pbckhl(nx,ny,nz))
c
            Endif
c
C
c       extinction and backscattering cross section (Hiro,Sep/02)
c
            kext_hyd = kextcw(nx,ny,nz) + kextrr(nx,ny,nz) + 
     &                 kextci(nx,ny,nz) + kextsn(nx,ny,nz) + 
     &                 kextgr(nx,ny,nz) + kexthl(nx,ny,nz)
c
            kexttot(nx,ny,nz)  = atm_ext(nx,ny,nz) + kext_hyd
            sbacktot(nx,ny,nz) = 
     &          pbckcw(nx,ny,nz)*salbcw(nx,ny,nz)*kextcw(nx,ny,nz) + 
     &          pbckrr(nx,ny,nz)*salbrr(nx,ny,nz)*kextrr(nx,ny,nz) + 
     &          pbckci(nx,ny,nz)*salbci(nx,ny,nz)*kextci(nx,ny,nz) + 
     &          pbcksn(nx,ny,nz)*salbsn(nx,ny,nz)*kextsn(nx,ny,nz) +  
     &          pbckgr(nx,ny,nz)*salbgr(nx,ny,nz)*kextgr(nx,ny,nz) +  
     &          pbckhl(nx,ny,nz)*salbhl(nx,ny,nz)*kexthl(nx,ny,nz)
c
 220     Continue
c
c simplified treatment of melting particles
c only for stratiform rain
c
c "nzfrz.gt.1" was inserted by Hiro (May/2007)
c
         If ( ics(nx,ny) .eq. 2 .and. nzfrz(nx,ny).gt.1 ) then

         Do 230 nz = nzfrz(nx,ny)-1, nzfrz(nx,ny)
c
            kextml(nx,ny,nz) = 0.
            salbml(nx,ny,nz) = 0.
            pbckml(nx,ny,nz) = 0.
c
            If ( lut ) then
c
            Call mie_lut (273.,rain_avg(nx,ny,nz)+snow_avg(nx,ny,nz),
     &                    kextml_lut,salbml_lut,pbckml_lut,
     &                    273.,pts_hydro,
     &                    1,npts_hydro,1,mxpts_hydro,
     &                    kextml0,salbml0,pbckml0)
c              
            Else
c
            Tavg = 0.5*(temp_lev(nx,ny,nz-1)+temp_lev(nx,ny,nz))
            Call df_mie_melt(freqcy, Tavg, rain_avg(nx,ny,nz),
     &                       snow_avg(nx,ny,nz),
     &                       kextml0,asymml0,salbml0,pbckml0)
c
            Endif
c
            If ( nz .eq. nzfrz(nx,ny) ) then
               kextml(nx,ny,nz) = f_frz*kextml0 + (1.-f_frz)*
     &                  (kextrr(nx,ny,nz) + kextsn(nx,ny,nz))
               If ( kextml(nx,ny,nz) .gt. 1.e-20)
     &         salbml(nx,ny,nz) = (f_frz*salbml0*kextml0 + (1.-f_frz)*
     &              (salbrr(nx,ny,nz)*kextrr(nx,ny,nz) + 
     &               salbsn(nx,ny,nz)*kextsn(nx,ny,nz))) / 
     &                                kextml(nx,ny,nz)
               If ( salbml(nx,ny,nz) .gt. 1.e-20 ) 
     &         pbckml(nx,ny,nz) = 
     &          (f_frz*pbckml0*salbml0*kextml0 + (1.-f_frz)*
     &          (pbckrr(nx,ny,nz)*salbrr(nx,ny,nz)*kextrr(nx,ny,nz) + 
     &           pbcksn(nx,ny,nz)*salbsn(nx,ny,nz)*kextsn(nx,ny,nz)))
     &                         / (salbml(nx,ny,nz)*kextml(nx,ny,nz))
            Endif
            If ( nz .eq. nzfrz(nx,ny)-1 ) then
               kextml(nx,ny,nz) = (1.-f_frz)*kextml0 + f_frz*
     &                  (kextrr(nx,ny,nz) + kextsn(nx,ny,nz))
               If ( kextml(nx,ny,nz) .gt. 1.e-20)
     &         salbml(nx,ny,nz) = ((1.-f_frz)*salbml0*kextml0 + f_frz*
     &              (salbrr(nx,ny,nz)*kextrr(nx,ny,nz) + 
     &               salbsn(nx,ny,nz)*kextsn(nx,ny,nz))) / 
     &                                kextml(nx,ny,nz)
               If ( salbml(nx,ny,nz) .gt. 1.e-20 ) 
     &         pbckml(nx,ny,nz) = 
     &              ((1.-f_frz)*pbckml0*salbml0*kextml0 + f_frz*
     &          (pbckrr(nx,ny,nz)*salbrr(nx,ny,nz)*kextrr(nx,ny,nz) + 
     &           pbcksn(nx,ny,nz)*salbsn(nx,ny,nz)*kextsn(nx,ny,nz)))
     &                         / (salbml(nx,ny,nz)*kextml(nx,ny,nz))
            Endif
c
            kexttot(nx,ny,nz) = atm_ext(nx,ny,nz) + kextml(nx,ny,nz) +
     &                          kextcw(nx,ny,nz) + kextci(nx,ny,nz) +
     &                          kextgr(nx,ny,nz) + kexthl(nx,ny,nz)
            sbacktot(nx,ny,nz) = 
     &           pbckcw(nx,ny,nz)*salbcw(nx,ny,nz)*kextcw(nx,ny,nz) +
     &           pbckci(nx,ny,nz)*salbci(nx,ny,nz)*kextci(nx,ny,nz) +  
     &           pbckml(nx,ny,nz)*salbml(nx,ny,nz)*kextml(nx,ny,nz) +
     &           pbckgr(nx,ny,nz)*salbgr(nx,ny,nz)*kextgr(nx,ny,nz) +
     &           pbckhl(nx,ny,nz)*salbhl(nx,ny,nz)*kexthl(nx,ny,nz)
c
 230     Continue
c
         Endif
c
 210    Continue
c
c       Only nadir profiles are currently simulated.
c
        write(*,*) '*** calculating dBZ ***'
        Call radar_echo(ngridx, ngridy, umu, nlyr, 
     $               temp_lev, hgt_lev, kexttot, sbacktot, freqcy,
     $               dBZ, PIA)
c
        Do nx = 1, ngridx
           Do ny = 1, ngridy
              Do nz = 1, nlyr
                 dBZ_out(nf,nx,ny,nz) = dBZ(nx,ny,nz)
              End do
              PIA_out(nf,nx,ny) = PIA(nx,ny)
           End do
        End do
c
 200  Continue
c
      Return
      End
