      Subroutine rd_CRM ( press_lev, temp_lev, tskin, ics, 
     &             rainrate_sfc, wind_lev, 
     &             hgt_lev, relhum_avg, cloud_water_avg, rain_avg, 
     &             cloud_ice_avg, snow_avg, graupel_avg, hail_avg, 
     &             ngridx, ngridy, nlyr, deltax, deltay, 
     &             wind_sfc, soilH2O, input_file )
      Implicit NONE
c
      Include 'MAINparameters.inc'
c
c Input: 
c  - file name 
c     input_file
c Outputs: 
c  - geophysical parameters and model grid dimensions
c     press_lev, temp_lev, tskin, ics, rainrate_sfc, wind_lev, hgt_lev, 
c     relhum_avg, cloud_water_avg, rain_avg, cloud_ice_avg, snow_avg, 
c     graupel_avg, hail_avg, ngridx, ngridy, nlyr, deltax, deltay, 
c     wind_sfc, soilH2O
c
      Integer i,j,ix,iy,k,irec,nchar,length_char
c
      character  input_file*(*)
      character  input_pathfile*90
      character  profile_name*20,pathandfile*100
c      
c I/O
c
      nchar = length_char ( input_file )
      input_pathfile = CRMdir//input_file(1:nchar)//'.bin'
      open(unit = 14, file = input_pathfile ,
     $     access = 'DIRECT' , recl = 44 )
c
      Write(*,*) 'Now processing on '//input_file(1:18)
c
      irec = 1
c
C Read CRM grid size
C  #s of grid points along x and y, # of vertical layers,
C  grid intervals along x and y (not in use)
C
      Read(14,rec=1) ngridx,ngridy,nlyr,deltax,deltay
c
C Read CRM profiles 
C
      do 20 i = 1, ngridx
        do 20 j = 1, ngridy
          irec = irec + 1
          Read(14,rec=irec) 
c
C             ### READ IN SURFACE PARAMETERS ###
c surface pressure in [hPa] and surface air temperature in [K]
c
     &                      press_lev(i,j,0), temp_lev (i,j,0),
c
c surface skin temperature in [K] and convective/stratiform index
c (ics=1 for convective and ics=2 for stratiform. The dielectric
c  parameterization of melting particles are applied to profiles w ics=2.)
c 
     &                      tskin    (i,j),   ics      (i,j),
c
c surface rain rate in [mm/hr] (not in use) and surface vertical wind 
c in [m/s] (not in use)
c
     &                      rainrate_sfc(i,j),wind_lev (i,j,0)
c
c surface height in [km]
c
          hgt_lev(0) = 0.
C
C             ### READ IN VERTICAL PROFILES ###
C Parameters with the suffix "_lev" are defined at layer interfaces, while
c parameters with the suffix "_avg" are at layer centers. The k-th layer is
c bound between the k-th and (k-1)-th interfaces.
c
          do k = 1, nlyr
            irec = irec + 1
            Read(14,rec=irec) 
c
c height in [km], pressure in [hPa], and air temperature in [K]
c
     &            hgt_lev(k), press_lev(i,j,k), temp_lev(i,j,k), 
c
c relative humidity in [%] and cloud water content in [g/m3]
c                           (= cloud mixing ratio * air density)
c
     &            relhum_avg(i,j,k), cloud_water_avg(i,j,k), 
c
c rain water content in [g/m3] and cloud ice content in [g/m3]
c
     &            rain_avg(i,j,k),   cloud_ice_avg(i,j,k), 
c
c snow content in [g/m3] and graupel content in [g/m3]
c
     &            snow_avg(i,j,k),   graupel_avg(i,j,k), 
c
c hail ice content in [g/m3] and vertical wind in [m/s] (not in use)
c
     &            hail_avg(i,j,k),   wind_lev(i,j,k)
c
          end do
c
 20   Continue
c
c Define near-surface wind (applicable when background="water") and 
c soil moisture (when background="land") for use by 'SIMmicro.f'.
c
      do 30 i = 1, ngridx
        do 30 j = 1, ngridy
c
           wind_sfc(i,j) = 6.
           soilH2O (i,j) = 0.
c
 30   Continue
c
      Return
      End
