      SUBROUTINE ABGAS1(V,AMTPB,AMTP,TAUM,TWGP,TX,AAA,CPSS)
C EXTINCTION COEFFICIENT OF THE MOLECULAR ABSORPTION/
C        GAS MIXING RATIO (PPMV)
C Associated routines with this routine: SCALD, TAUF, TRKF
C--- HISTORY
C 92. 6.30  CREATED
C    11.13  Make all subscript of arrays more than 0 by Masahito Tsukamoto.
C 93. 6. 8  Add TAUF
C 94. 3.26  ALOG -> LOG at several places of sub-routines.
C--- INPUT
C V        R       WAVENUMBER (CM-1)
C AMTPB   R(53)    EFFECTIVE GAS AMOUNT -DENSTY- INTEGRATED
C                  ALONG OPTICAL PATH, FOR BANDS DEFINED BY -ABCDTA-
C                  UNIT IS in PPMV. SEE -SCALD-
C--- OUTPUT
C V                TRUNCATED TO A MULTIPLE OF 5 CM-1
C AMTP    R(28)    SCALING FACTOR OF ABSORBER AMOUNT FOR LINE ABSORPTION
C                     1: H2O    2: CO2     3: O3      4: N2O     5: CO
C                     6: CH4    7: O2
C                     8: NO     9: SO2    10: NO2    11: NH3
C                     FIRST 11 ARE MEANINGFUL
C TX      R(28)    CONTINUUM ABSORPTION OPTICAL THICKNESS OF EACH GAS
C                     1, 3, 7, 11, 22 HAVE NON-ZERO VALUES
C TAUM   R(3,28)   MOLECULAR LINE ABSORPTION COEFF. (EACH GAS)
C                    PER UNIT OF -AMTPB-
C                    FIRST 11 HAVE NON-ZERO MEANINGFUL
C                    See TRKF and TAUF for how to use it.
C TWGP   R(3,28)   K DISTRIBUTION
C AAA    R(28)     NON-LINEAR PARAMETER FOR ABSORBER AMOUNT DEPENDENCE
C                    IN RANDOM MODEL TRANSMISSION
C CPSS   R(28)     LOG10(CONVERGENCE FACTOR OF ABSORPTION COEFF.
C                    TO REFERENCE PRESSURE AND TEMPERATURE)
C---NOTES
C USE -TRK- TO CALCULATE THE TRANSMITTANCE BY K-DISTRIBUTION
C USE -TRB- TO CALCULATE THE TRANSMITTANCE BY BAND MODEL
C
C USE -DBLTX- TO CALCULATE TRANSMITTANCE BY RANDOM MODEL
C LINK CXDTA      LOCATES COFFICIENT FOR DOUBLE EXPONENTIAL
C      C4DAT      N2 CONTINUUM ABSORPTION COFFICIENT
C      SLF296     SELF-BROADENED WATER VAPOR CONTINUUM AT 296K
C      SLF260     SELF-BROADENED WATER VAPOR CONTINUUM AT 260K
C      FRN296N    FOREIGN-BROADENED WATER VAPOR CONTINUUM AT 296K
C      FUDGE
C      ABCDTA     MOVE DOUBLE EXPONENTIAL COEFFICIENTS INTO NEW ARRAYS
C      C8DTA      RETURN CHAPPUIS VISIBLE ABSORPTION COEFFICIENT
C      O2CONT     O2 CONTINUUM FOR 1395-1760 CM-1
C      SCHRUN     UV O2 SCHUMANN-RUNGE BAND MODEL PARAMETERS
C      O3HHT0     UV O3 HARTLEY BAND TEMPERATURE DEPENDENT COFFICIENT
C                 CONSTANT TERM (24370 TO 40800 CM-1)
C      O3HHT1     UV O3 HARTLEY BAND TEMPERATURE DEPENDENT COFFICIENT
C                 LINEAR TERM
C      O3HHT2     UV O3 HARTLEY BAND TEMPERATURE DEPENDENT COFFICIENT
C                 QUADRATIC TERM
C      O3UV       UV O3 INTERPOLATION FOR 40800-54054 CM-1
C
CC***********************************************************************
C
      SAVE
c
      Include 'VISIRparameters.inc'
c
CCC      PARAMETER (KNM=28)
      DIMENSION AMTPB(53),TX(KNM),TAUM(3,KNM),TWGP(3,KNM),AMTP(KNM)
     & ,AAA(KNM),CPSS(KNM)
      DIMENSION ABB(63),CP1S(11),FAC(3),GKWJ(3,11),DPWJ(3,11)
      COMMON /H2O/    CPH2O(3515)
      COMMON /O3/     CPO3 ( 447)
      COMMON /UFMIX1/ CPCO2(1219)
      COMMON /UFMIX2/ CPCO ( 173),CPCH4( 493),CPN2O( 704),CPO2 ( 382)
      COMMON /TRACEG/ CPNH3( 431), CPNO(  62),CPNO2( 142),CPSO2( 226)
      COMMON /WNLOHI/
     L   IWLH2O(15),IWLO3 ( 6),IWLCO2(11),IWLCO ( 4),IWLCH4( 5),
     L   IWLN2O(12),IWLO2 ( 7),IWLNH3( 3),IWLNO ( 2),IWLNO2( 4),
     L   IWLSO2( 5),
     H   IWHH2O(15),IWHO3 ( 6),IWHCO2(11),IWHCO ( 4),IWHCH4( 5),
     H   IWHN2O(12),IWHO2 ( 7),IWHNH3( 3),IWHNO ( 2),IWHNO2( 4),
     H   IWHSO2( 5)
      COMMON /AABBCC/ AA(11),BB(11),CC(11),IBND(11),A(11),CPS(11)
      DATA   CF1/3.159E-08/,CF2/2.75E-04/
      DATA FAC /1.0,0.09,0.015/
      BIGEXP=30.
C
      DO 2 M=1,KNM
      AAA(M)=0
      CPSS(M)=0
    2 TX(M)=0
      DO 7 K=1, 3
      DO 7 M= 1,KNM
      TAUM(K,M)=0
    7 TWGP(K,M)=0
C
      ALPHA2=200.**2
      FACTOR=0.5
      IV=V/5.
      IV=IV*5
      INDH2O=1
      INDO3 =1
      INDCO2=1
      INDCO =1
      INDCH4=1
      INDN2O=1
      INDO2 =1
      INDNH3=1
      INDNO =1
      INDNO2=1
      INDSO2=1
      V=FLOAT(IV)
      CALL CXDTA(CP,V,IWLH2O,IWHH2O,CPH2O,INDH2O)
      CPS(1) = CP
      CALL CXDTA(CP,V,IWLCO2,IWHCO2,CPCO2,INDCO2)
      CPS(2) = CP
      CALL CXDTA(CP,V,IWLO3, IWHO3, CPO3, INDO3 )
      CPS(3) = CP
      CALL CXDTA(CP,V,IWLN2O,IWHN2O,CPN2O,INDN2O)
      CPS(4) = CP
      CALL CXDTA(CP,V,IWLCO, IWHCO, CPCO, INDCO )
      CPS(5) = CP
      CALL CXDTA(CP,V,IWLCH4,IWHCH4,CPCH4,INDCH4)
      CPS(6) = CP
      CALL CXDTA(CP,V,IWLO2, IWHO2, CPO2, INDO2 )
      CPS(7) = CP
      CALL CXDTA(CP,V,IWLNO, IWHNO, CPNO, INDNO )
      CPS(8) = CP
      CALL CXDTA(CP,V,IWLSO2,IWHSO2,CPSO2,INDSO2)
      CPS(9) = CP
      CALL CXDTA(CP,V,IWLNO2,IWHNO2,CPNO2,INDNO2)
      CPS(10)= CP
      CALL CXDTA(CP,V,IWLNH3,IWHNH3,CPNH3,INDNH3)
      CPS(11)= CP
      CALL ABCDTA(IV)
C K DISTRIBUTION
      DO  1 M = 1,11
      AAA(M)=A(M)
      CPSS(M)=CPS(M)
      IB = IBND(M)
C 92.11.13 Modified by M.T.
C     IB = IBND(M)
      IF(IB.LE.0)THEN
	AMTP(M)=0
       ELSE
        AMTP(M)=AMTPB(IB)
      END IF
C Correction end 92.11.13
  1   CP1S(M)= 10.**CPS(M)
      DO  4  M = 1,11
      DO  5  K = 1,3
      IW = IBND(M)
      GKWJ(K,M) = 0.
      DPWJ(K,M) = 0.
      IF(IW. LE. 0) GO TO 5
      GKWJ(K,M) = FAC(K) * CC(M)
      IF(K .EQ. 1) DPWJ(K,M) = AA(M)
      IF(K .EQ. 2) DPWJ(K,M) = BB(M)
      IF(K .EQ. 3) DPWJ(K,M) = 1- AA(M) - BB(M)
  5   CONTINUE
  4   CONTINUE
C
      CALL C4DTA(ABB(45),V)
C WATER CONTINUUM
       CALL  SLF296(V,SH2OT0)
       CALL  SLF260(V,SH2OT1)
       CALL  FRN296(V,FH2O)
       T0=296.
       T1=260.
       IF(SH2OT0   .GT.0.) THEN
C CORRECTION TO SELF CONTINUUM (1 SEPT 85); FACTOR OF 0.78 @ 1000
          XH2O=      (1. - 0.2333*( ALPHA2/((V -1050.)**2+ALPHA2)) )
          SH2OT0  =   SH2OT0 *XH2O
          SH2OT1  =   SH2OT1 *XH2O
      END IF
C PROTECT AGAINST EXPONENTIAL UNDERFLOW AT HIGH FREQUENCY
      VTEMP=V/.6952
      IF(VTEMP/T1.LE.BIGEXP)THEN
           XD=EXP(-V/(T0*0.6952))
           RADFN0=V*(1.-XD)/(1.+XD)
           XD=EXP(-V/(T1*0.6952))
           RADFN1=V*(1.-XD)/(1.+XD)
      ELSE
           RADFN0 = V
           RADFN1 = V
      ENDIF
      CALL FUDGE(V,FDG)
      ABB(42)=SH2OT0*RADFN0
      CALL C8DTA(ABB(46),V)
      ABB(43)=(SH2OT1*RADFN1)-(SH2OT0*RADFN0)
      ABB(44)=(FH2O+FDG)*RADFN0
C HNO3 ABSORPTION CALCULATION
C      CALL HNO3 (V,ABB(47))
      CALL HERTDA(ABB(48),V)
      CALL O2CONT(V,SIGO20,SIGO2A,SIGO2B)
      IF(V. GT. 49600 )CALL SCHRUN(V,CPS(7))
C DIFFUSE OZONE
           ABBUV = 0.
           IF(V .GT.24370. .AND. V .LT.40800.) THEN
              ABB(46) = 0.
              CALL O3HHT0                (V,C0)
              CALL O3HHT1                (V  ,CT1)
              CALL O3HHT2                (V,  CT2)
C
      END IF
      IF(V .GE.40800)   THEN
           CALL O3UV(V,C0)
           ABB(46) = 0.
      ENDIF
      WO3  = AMTPB(46) * .269
      IF(V .GT.24370. .AND. V .LT.40800.) THEN
           W1O3 = AMTPB(52)
           W2O3 = AMTPB(53)
C   COZ   = C0  *(1.+ CT1  *TC+CT2   *TC*TC)
           ABBUV = C0 * (WO3+CT1*W1O3+CT2   *W2O3)
      ENDIF
      IF(V .GE.40800) THEN
           ABBUV = C0 *  WO3
      ENDIF
C WATER VAPOR CONTINUUM
      TXH2O1 = ABB(42) *(AMTPB(42) *1.0E-20)
      TXH2O2 = ABB(43) *(AMTPB(43) *1.0E-20)
      TXH2O3 = ABB(44) *(AMTPB(44) *1.0E-20)
      TX(1) = TXH2O1 + TXH2O2 + TXH2O3
C N2 CONTINUUM
      TX(22)=ABB(45)*AMTPB(45)
C     OZONE
      TX(3) = ABB(46)*AMTPB(46) + ABBUV
C OXYGEN
      WT2 =   AMTPB(48) - AMTPB(50)*220.
      TX(7) =SIGO20*(AMTPB(50)+SIGO2A*WT2+SIGO2B*AMTPB(49))
C UV O2 HERZBERG CONTINUUM ABSORPTION CALCULATION
      TX(7) =AMTPB(51)*ABB(48)+ TX(7)
C HNO3
      TX(11)=AMTPB(47)*ABB(47)
C MOLECULAR LINE ABSORPTION
C   EVALUATE THE WEIGHTED K DISTRIBUTION QUANTITIES FOR
C   WATER VAPOR AND THE UNIFORMLY MIXED GASES
      DO 8 K=1, 3
      DO 8 M= 1,11
      TAUM(K,M)=GKWJ(K,M)*CP1S(M)
    8 TWGP(K,M)=DPWJ(K,M)*TAUM(K,M)
      RETURN
      END
