      SUBROUTINE FTRN21(INDG,INIT,M,INDA,INDT,IMTHD,NA1U,AM1U,NA0,AM0
     &,NDA,AMUA,WA,NLN,NLGN1,G,DPT,OMG,NPLK1,CPLK,GALB,BGND
     &,FSOL,NTAU,UTAU,SCR,SCI,FLXD,FLXU,AI,ERR)
C
C SOLVE THE RADIATIVE TRANSFER IN ATMOSPHERE SYSTEM FOR EACH FOURIER
C COMPONENT.
C  -DOM AND ADDING METHOD-
C BY TERUYUKI NAKAJIMA
C SINCE THIS SYSTEM DOES NOT INCLUDE A OCEAN SURFACE,
C KNLT=KNLN IN TRN1.
C
C--- HISTORY
C 89.11. 2 CREATED FROM STRN7 WITH THERMAL EMISSION
C 90.12. 1 CALL AINT(M,L,  ->  CALL AINT(M,LT,
C 92.12.   ADD OCEAN SURFACE
C 93. 1.   CHANGE UPWARD ADDING
C            USE CSPL1 AND CSPLI
C     3.10 BUG ( XX,YY USED K1 )
C     4. 5 NSF = 1 for initialization (Terry)
C     5. 4 INDG instead of NSF and LOSW
C          Subtracted single scattering if INDG>0
C 94. 5. 7 Bug for interpolating AIB(J)
C 95. 5.25 Replace GRNDL by GRNDL3
C 95. 6. 2 With GRNDO1 and with SCR and SCI
C 96. 3.17 Add ERT By Takashi Nakajima to debug thermal.
C 02.05.14 AINT->AINTEG
C--- INPUT
C INDG       I       0: Lambert surface
C                    1: Ocean surface initialized
C                    2: Ocean surface with no-initialization
C                    When INDG>0 and IMTHD>0 then single scattering correction
C                      for ocean surface reflection
C INIT       I       1: INITIALIZE THE PART (DEPENDENT AMUA, UTAU)
C                    0:  BYPASS THE M-INDEPENDENT PART.
C M          I       FORIER ORDER.
C INDA       I       0: FLUX ONLY.
C                    1: INTENSITY USING AM1U.
C                    2: NA1U AND AM1U ARE SUPPOSED TO BE
C                       2*NDA AND (-AMUA, +AMUA).
C INDT       I       0: SET USER DEFINED DEPTH.
C                    1: SAME AS ABOVE.
C                    2: SET NTAU AND UTAU AS NLN1 AND DPT.
C IMTHD      I      -1: NT,  0: DMS-METHOD  FOR INTENSITY/FLUX
C                    1: MS,  2:TMS,  3:IMS-METHOD FOR INTENSITY.
C                    When INDG>0 and IMTHD>0 then single scattering correction
C                      for ocean surface reflection
C NA1U       I       NUMBER OF EMERGENT ZENITH ANGLES IN THE SPHERE.
C                      NA1U=2*NDA WHEN INDA=2.
C AM1U    R(KNA1U)   CONSINE OF THE EMERGENT ZENITH ANGLES.
C                      + FOR DOWNWARD, - FOR UPWARD.
C                      AM1U = (-AMUA, +AMUA) WHEN INDA=2.
C NA0        I       NUMBER OF THE SOLAR INCIDENCES.
C AM0     R(NA0)     CONSINE OF SOLAR ZENITH ANGLES .GT.0.
C NDA        I       NO. OF ZENITH-QUADRATURE ANGLES IN THE HEMISPHERE.
C AMUA    R(KNDM)    COSINE OF THE ZENITH-QUADRATURE ANGLES. 1 TO 0.
C WA      R(KNDM)    CORRESPONDING QUADRATURE WEIGHTS.
C NLN        I       NUMBER OF ATMOSPHERIC SUBLAYERS.
C NLGN1   I(KNLN)    MAXIMUM ORDER OF THE LEGENDRE SESIES OF THE PHASE
C                      FUNCTION + 1
C G     R(KLGN1,     LEGENDRE MOMENTS OF PHASE FUNCTION.
C           KNLN)      G(1,L)=1
C DPT     R(KNLN1)   OPTICAL DEPTH AT THE INTERFACES BETWEEN SUBLAYERS.
C                      TOP TO BOTTOM (TOP = 0 FOR NORMAL APPLICATION).
C OMG     R(KNLN)    SINGLE SCATTERING ALBEDO.
C NPLK1      I       NUMBER OF ORDER TO APPROXIMATE PLANK + 1.
C                      IF 0 THEN NO THERMAL.
C CPLK    R(KPLK1    PLANK FUNCTION (B) =
C         ,KNLN)       SUM(IB=1,NPLK1) CPLK(IB,L) TAU**(IB-1).
C                      TAU IS OPTICAL DEPTH MEASURED FROM
C                      THE TOP OF THE SUBSURFACE.
C GALB       R       Ground albedo if INDG=0
C                    U10 (m/sec)   if INDG>0
C BGND       R       (1-GALB)*B when INDG=0
C                    B          when INDG>0
C                    where B is Plank function
C                      (SAME UNIT AS FSOL).
C FSOL       R       SOLAR IRRADIANCE (W/M2/MICRON).
C NTAU       I       NUMBER OF USER DEFINED OPTICAL DEPTHS.
C                      NTAU=NLN+1 WHEN INDT=2.
C UTAU    R(KNTAU)   OPTICAL DEPTHS WHERE THE FIELD IS CALCULTED.
C                      TOP TO BOTTOM.
C                      UTAU=DPT WHEN INDT=2.
C--- OUTPUT
C INDG               if 1 then 2
C INIT               0
C FLXD  R(KNA0,KNTAU) DOWNWARD FLUX AT UTAU.
C FLXU               SAME AS FLXD BUT FOR UPWARD FLUX.
C AI      R(KNA1U,   I(MU1U(I), MU0(J), L)
C           KNA0,    INTENSITY AT UTAU.
C           KNTAU)   Subtracted single scattering if INDG>0
C ERR      C*64      ERROR INDICATER.
C--- CORRESPONDENCE BETWEEN VARIABLES AND PARAMETERS
C KNA1U       NA1U
C KNA1        NA1
C KNA0        NA0
C KNDM        NDA
C KNLN        NLN
C KNLN1       KNLN+1
C KNTAU       NTAU
C KLGN1       NLGN1
C KPLK1       NPLK1
C--- NOTES FOR LOCAL VARIABLES
C NA1        I       NUMBER OF STREAMS FOR AM1.
C AM1     R(KNA1)    ABS(AM1U).
C IIAM1   I(KNA1U)   DIRECTION NUMBER OF AM1 FOR EACH AM1U.
C                      + FOR DOWNWARD, - FOR UPWARD.
C IITAU   I(KNTAU)   SUBLAYER NUMBER FOR USER DEFINED DEPTHS.
C$ENDI
      SAVE EPS,WMP,WMM,IITAU
C PARAMETERS
C
      include 'VISIRparameters.inc'
c
c      PARAMETER (KNA1U =10)
c      PARAMETER (KNA1  =10)
c      PARAMETER (KNA0  =10)
c      PARAMETER (KNDM  =10)
c      PARAMETER (KNLN  =50)
c      PARAMETER (KNTAU =51)
c      PARAMETER (KPLK1 =2)
C
c      PARAMETER (KNLN1=KNLN+1,KLGN1=2*KNDM)
      Parameter ( KLGN2=2*KNDM )
      PARAMETER (PI=3.141592654,RAD=PI/180.0)
C AREAS FOR THIS ROUTINE
      CHARACTER ERR*(*)
      DIMENSION AM1U(KNA1U),AM0(KNA0),AMUA(KNDM),WA(KNDM)
     & ,NLGN1(KNLN),G(KLGN2,KNLN),DPT(KNLN1),OMG(KNLN)
     &,CPLK(KPLK1,KNLN),UTAU(KNTAU),FLXD(KNA0,KNTAU),FLXU(KNA0,KNTAU)
     &,AI(KNA1U,KNA0,KNTAU)
C WORKING AREAS
      DIMENSION AM1(KNA1),WMP(KNDM),WMM(KNDM)
     &,PL1(KNA1,KLGN2),PL0(KNA0,KLGN2),PLA(KNDM,KLGN2),GBUF(KLGN2)
     &,PT(KNDM,KNDM),PR(KNDM,KNDM),PT0(KNDM,KNA0),PR0(KNDM,KNA0)
     &,PR1(KNA1,KNDM),PT1(KNA1,KNDM),PR10(KNA1,KNA0),PT10(KNA1,KNA0)
     &,ALFA(KNDM,KNA0),BETA(KNDM,KNA0),UUP(KNDM,KNA0),UDN(KNDM,KNA0)
     &,AII(KNA1U,KNA0,KNLN1),AIB(KNA0),IIAM1(KNA1U),IITAU(KNTAU),CCP(3)
C FOR HOMOG2
      DIMENSION CPL(KPLK1),R(KNDM,KNDM),T(KNDM,KNDM)
     &,ER(KNDM,KNA0),ET(KNDM,KNA0),ZEIG(KNDM)
     &,Q(KNDM,KNDM),QI(KNDM,KNDM),C11(KNDM,KNDM),C22(KNDM,KNDM)
     &,VP(KNDM,KNA0),VM(KNDM,KNA0),DP(KNDM,KPLK1),DM(KNDM,KPLK1)
C
      DIMENSION ERT(KNDM,KNA0)
C FOR TRN1 (INCLUDE GROUND AS A SUBLAYER)
C      PARAMETER (KNLNM=KNLN1,KNLNM1=KNLNM+1,KNLTM=KNLN1)
      DIMENSION IUP(KNLNM),IDN(KNLNM),NDD(KNLNM1)
     &, RE(KNDM,KNDM,KNLTM),  TE(KNDM,KNDM,KNLTM)
     &,SER(KNDM,KNA0,KNLNM),SET(KNDM,KNA0,KNLNM)
     &,RUP(KNDM,KNA0,KNLNM1),RDN(KNDM,KNA0,KNLNM1)
C FOR INTENSITY INTERPOLATION IN MULTI-LAYER SYSTEM
      DIMENSION QE(KNDM,KNDM,KNLN),QIE(KNDM,KNDM,KNLN)
     &,C1E(KNDM,KNDM,KNLN),C2E(KNDM,KNDM,KNLN),ZEE(KNDM,KNLN)
     &,DPE(KNDM,KPLK1,KNLN),DME(KNDM,KPLK1,KNLN)
     &,VPE(KNDM,KNA0,KNLN),VME(KNDM,KNA0,KNLN)
C SET AND CLEAR VARIABLES
      DIMENSION XX(KNDM),YY(KNDM),A(KNDM),B(KNDM),C(KNDM),D(KNDM)
      ERR=' '
      IF(INIT.GT.0) THEN
        INIT=0
        CALL CPCON(CCP)
        EPS=CCP(1)*10
        DO 1 I=1,NDA
        WMP(I)=SQRT(WA(I)*AMUA(I))
    1   WMM(I)=SQRT(WA(I)/AMUA(I))
C SET SUBLAYER WHERE USER-DEFINED DEPTH RESIDES -IITAU-.
        DO 40 IT=1,NTAU
        ODP=UTAU(IT)
        EPS1=ODP*EPS
        DO 41 L=1,NLN
          IF((ODP-DPT(L))*(ODP-DPT(L+1)).LE.0.0) GOTO 42
   41   CONTINUE
        ERR='OUT OF BOUNDS IN LAYER SETTING'
        RETURN
   42   IF(ABS(ODP-DPT(L)).LE.EPS1) THEN
          IITAU(IT)=-L
         ELSE
          IF(ABS(ODP-DPT(L+1)).LE.EPS1) THEN
            IITAU(IT)=-(L+1)
           ELSE
            IITAU(IT)=L
          ENDIF
        ENDIF
   40   CONTINUE
      ENDIF
C SET AM1, IIAM1
      IF(INDA.GT.0) THEN
        NA1=1
        AM1(1)=ABS(AM1U(1))
        IIAM1(1)= 1
        IF(NA1U.GE.2) THEN
          DO 43 IU=2,NA1U
          X=ABS(AM1U(IU))
          DO 44 J=1,NA1
          IF(ABS(X-AM1(J)).LE.EPS) THEN
            IIAM1(IU)= J
            GOTO 43
          ENDIF
   44     CONTINUE
          NA1=NA1+1
          IF(NA1.GT.KNA1) THEN
            ERR='SETTING ERROR OF AM1U'
            RETURN
          ENDIF
          AM1(NA1)=X
          IIAM1(IU)= NA1
   43     CONTINUE
        ENDIF
      ENDIF
C CHECK MAXIMUM NLGN1
      MXLGN1=0
      DO 2 L=1,NLN
    2 MXLGN1=MAX(MXLGN1,NLGN1(L))
      IF(MXLGN1.LE.0) THEN
        ERR='NLGN1 ARE ALL ZERO'
        RETURN
      ENDIF
      M1=M+1
C SET LEGENDRE POLYNOMIALS.
      CALL PLGND(M1,MXLGN1,NA0,KNA0,AM0 ,PL0)
      CALL PLGND(M1,MXLGN1,NDA,KNDM,AMUA,PLA)
C SCALING FOR SYMMETLICITY
        DO 5 I=1,NDA
        DO 5 J=1,MXLGN1
    5   PLA(I,J)=WMM(I)*PLA(I,J)
C SOLVE THE EIGENVALUE PROBLEM OF ATMOSPHERIC SUBLAYERS.
      DO 6 L=1,NLN
        LB=L
        NAL1=NLGN1(L)
        NDD(L)=NDA
        IUP(L)=L
        IDN(L)=L
        T1=DPT(L)
        T2=DPT(L+1)
        W=OMG(L)
        IF(NPLK1.GT.0) THEN
          DO 7 I=1,NPLK1
    7     CPL(I)=2*PI*(1-W)*CPLK(I,L)
        ENDIF
CC SCATTERING MEDIA
CC PT, PR
        CALL EQ12(GBUF,G,NAL1,LB,KLGN2)
        CALL PHAS2(M1,NAL1,NDA,NDA,KNDM,KNDM,KNDM,PT,PR
     &     ,GBUF,PLA,PLA)
CC PT0, PR0
        CALL PHAS2(M1,NAL1,NDA,NA0,KNDM,KNDM,KNA0,PT0,PR0
     &     ,GBUF,PLA,PL0)
CC EIGENVALUE PROBLEM (REPLACED HOMOG2 with HOMOG3)
        CALL HOMOG3(M,T1,T2,W,NDA,AMUA,WMM,NA0,AM0
     &      ,PR,PT,PR0,PT0,FSOL,NPLK1,CPL,R,T,ER,ET,ZEIG
     &      ,Q,QI,C11,C22,VP,VM,DP,DM,ERR)
        IF(ERR.NE.' ') RETURN
        CALL EQ32(RE ,R  ,NDA,NDA,LB,KNDM,KNDM,KNLTM,KNDM,KNDM)
        CALL EQ32(TE ,T  ,NDA,NDA,LB,KNDM,KNDM,KNLTM,KNDM,KNDM)
        CALL EQ32(SER,ER ,NDA,NA0,LB,KNDM,KNA0,KNLNM,KNDM,KNA0)
        CALL EQ32(SET,ET ,NDA,NA0,LB,KNDM,KNA0,KNLNM,KNDM,KNA0)
C STORE EXTRA FOR INTERNAL FIELD AT ARBITRARY DEPTH.
        IF(INDT.NE.2 .OR. INDA.NE.2) THEN
          CALL EQ32(QE ,Q  ,NDA,NDA,LB,KNDM,KNDM,KNLN,KNDM,KNDM)
          CALL EQ32(QIE,QI ,NDA,NDA,LB,KNDM,KNDM,KNLN,KNDM,KNDM)
          CALL EQ32(C1E,C11,NDA,NDA,LB,KNDM,KNDM,KNLN,KNDM,KNDM)
          CALL EQ32(C2E,C22,NDA,NDA,LB,KNDM,KNDM,KNLN,KNDM,KNDM)
          CALL EQ32(VPE,VP ,NDA,NA0,LB,KNDM,KNA0,KNLN,KNDM,KNA0)
          CALL EQ32(VME,VM ,NDA,NA0,LB,KNDM,KNA0,KNLN,KNDM,KNA0)
          CALL EQ21(ZEE,ZEIG,NDA,LB,KNDM)
          IF(NPLK1.GT.0) THEN
            CALL EQ32(DPE,DP,NDA,NPLK1,LB,KNDM,KPLK1,KNLN,KNDM,KPLK1)
            CALL EQ32(DME,DM,NDA,NPLK1,LB,KNDM,KPLK1,KNLN,KNDM,KPLK1)
          ENDIF
        ENDIF
    6 CONTINUE
C LAMBERT SURFACE
      NLN1=NLN+1
      IF(INDG.LT.0) THEN
        NLT=NLN
       ELSE
        NLT=NLN1
        NDD(NLT)=NDA
        IUP(NLT)=NLT
        IDN(NLT)=NLT
        T1=DPT(NLT)
        IF(INDG.LE.0)THEN
          CALL GRNDL3(FSOL,GALB,BGND,T1,M,NDA,AMUA,WA,NA0,AM0
     &       ,R,T,ER,ET)
         ELSE
          CALL GRNDO1(INDG,FSOL,GALB,BGND,T1,M,NDA,WMP,NA0,AM0
     &     ,SCR,SCI,R,T,ER,ERT,ET,AMUA,WA)
        ENDIF
        CALL EQ32(RE , R,NDA,NDA,NLT,KNDM,KNDM,KNLTM,KNDM,KNDM)
        CALL EQ32(TE , T,NDA,NDA,NLT,KNDM,KNDM,KNLTM,KNDM,KNDM)
        CALL EQ32(SER,ER,NDA,NA0,NLT,KNDM,KNA0,KNLNM,KNDM,KNA0)
        CALL EQ32(SET,ET,NDA,NA0,NLT,KNDM,KNA0,KNLNM,KNDM,KNA0)
      ENDIF
      NLT1=NLT+1
      NDD(NLT1)=NDA
C ADDING OF THE SUBLAYERS.
      CALL TRN1(NLT,NDD,NA0,IUP,IDN,RE,TE,SER,SET,RUP,RDN,ERR)
      IF(ERR.NE.' ') RETURN
      IF(INDT.EQ.2 .AND. INDA.NE.1) THEN
        IF(INDA.EQ.2) THEN
          DO 8 L=1,NLN1
          DO 8 J=1,NA0
          IF(IMTHD.LE.0 .OR. INDG.LE.0) THEN
            DO 9 I=1,NDA
    9       AI(I,J,L)=RUP(I,J,L)/WMP(I)
           else
            DO 59 I=1,NDA
C Don't change RUP itself, because it is used for flux calculation later
C 97.3.17 Debug By Takashi Nakajima
C            RUP1=RUP(I,J,L)-SER(I,J,NLN1)
C     &                     *EXP(-(DPT(NLN1)-DPT(L))/AMUA(I))
            RUP1=RUP(I,J,L)+(ERT(I,J)-SER(I,J,NLN1))
     &                     *EXP(-(DPT(NLN1)-DPT(L))/AMUA(I))
C
   59       AI(I,J,L)=RUP1/WMP(I)
          ENDIF
          DO 10 I=1,NDA
   10     AI(NA1U+1-I,J,L)=RDN(I,J,L)/WMP(I)
    8     CONTINUE
        ENDIF
        IF(M.EQ.0) THEN
          DO 11 L=1,NLN1
          DO 11 J=1,NA0
          FU1=0
          FD1=0
          EX1=-UTAU(L)/AM0(J)
          TRNS0=EXPFN(EX1)*FSOL
CC FOR SCALED INTENSITY
          DO 12 I=1,NDA
          FU1=FU1+WMP(I)*RUP(I,J,L)
   12     FD1=FD1+WMP(I)*RDN(I,J,L)
          FLXU(J,L)=FU1
          FLXD(J,L)=FD1+TRNS0*AM0(J)
   11     CONTINUE
        ENDIF
        RETURN
      ENDIF
C--- INTERPOLATION OF THE FIELD.
      IF(INDA.EQ.1) CALL PLGND(M1,MXLGN1,NA1,KNA1,AM1 ,PL1)
      DO 13 L=1,NLN
        LT=L
        LB=LT+1
        DPTH=DPT(LT)
        TAU=DPT(LB)-DPTH
        T1=DPT(LT)
        T2=DPT(LB)
        W=OMG(L)
C INTEGRAL CONSTANTS:  ALFA, BETA
        CALL CINGR(NDA,NA0,LT,AM0,TAU,RDN,RUP,VPE,VME,C1E,C2E
     &   ,NPLK1,DPE,DME,ALFA,BETA)
C FLUXES OR INTENSITY WHEN INDA=2
        IF(M.EQ.0 .OR. INDA.EQ.2) THEN
          DO 14 IT=1,NTAU
          L1=IABS(IITAU(IT))
          IF(L1.EQ.NLN1) L1=NLN
          IF(L1.NE.L) GOTO 14
          DTAU=UTAU(IT)-DPTH
          IF(IITAU(IT).GT.0) THEN
CC IN THE SUBLAYER
            CALL ADISC(M,L1,NDA,NA0,AM0,WMP,TAU,DTAU,ZEE,QE,QIE
     &      ,VPE,VME,ALFA,BETA,NPLK1,DPE,DME,UDN,UUP)
           ELSE
CC JUST INTERFACE
            LU=IABS(IITAU(IT))
            DO 15 J=1,NA0
            DO 15 I=1,NDA
            UDN(I,J)=RDN(I,J,LU)/WMP(I)
   15       UUP(I,J)=RUP(I,J,LU)/WMP(I)
          ENDIF
CC INTENSITY WHEN INDA=2
          IF(INDA.EQ.2) THEN
            DO 18 J=1,NA0
            IF(IMTHD.LE.0 .OR. INDG.LE.0) THEN
              DO 16 I=1,NDA
   16         AI(I,J,IT)=UUP(I,J)
             ELSE
              DO 56 I=1,NDA
C Don't change UUP itself, because it is used for flux calculation later
C 97.3.17 Debug By Takashi Nakajima
C   56         AI(I,J,IT)=UUP(I,J)-SER(I,J,NLN1)/WMP(I)
C     &                  *EXP(-(DPT(NLN1)-UTAU(IT))/AMUA(I))
   56         AI(I,J,IT)=UUP(I,J)+(ERT(I,J)-SER(I,J,NLN1)/WMP(I))
     &                  *EXP(-(DPT(NLN1)-UTAU(IT))/AMUA(I))
C
            ENDIF
            DO 17 I=1,NDA
   17       AI(NA1U+1-I,J,IT)=UDN(I,J)
   18       CONTINUE
          ENDIF
CC FLUX
          IF(M.EQ.0) THEN
            DO 19 J=1,NA0
            FU1=0
            FD1=0
            EX1=-UTAU(IT)/AM0(J)
            TRNS0=EXPFN(EX1)*FSOL
            DO 20 I=1,NDA
            FU1=FU1+AMUA(I)*WA(I)*UUP(I,J)
   20       FD1=FD1+AMUA(I)*WA(I)*UDN(I,J)
            FLXU(J,IT)=FU1
   19       FLXD(J,IT)=FD1+TRNS0*AM0(J)
          ENDIF
   14     CONTINUE
        ENDIF
C--- INTENSITIES IN THE USER DEFINED DIRECTIONS (WITHOUT CONTRIBUTION
C     FROM THE INTERFACES).
C PHASE FUNCTIONS FOR ANGULAR INTERPOLATION
        IF(INDA.EQ.1) THEN
          NAL1=NLGN1(L)
          CALL EQ12(GBUF,G,NAL1,LT,KLGN2)
C PT10, PR10
          CALL PHAS2(M1,NAL1,NA1,NA0,KNA1,KNA1,KNA0,PT10,PR10
     &     ,GBUF,PL1,PL0)
C PT1, PR1
          CALL PHAS2(M1,NAL1,NA1,NDA,KNA1,KNA1,KNDM,PT1,PR1
     &     ,GBUF,PL1,PLA)
          DO 21 IU=1,NA1U
C AT INTERFACE.
          I=IIAM1(IU)
          IF(AM1U(IU).GT.0.0) THEN
CC DOWNWARD INCREMENT AT BOTTOM OF SUBLAYER (STAMNES INTEGRATION)
            CALL AINTEG(M,LT,NDA,NA0,I,FSOL,AM1U(IU),AM0,W,T2,T1,T2
     &       ,ZEE,QE,QIE,VPE,VME,PT10,PR10,PT1,PR1,ALFA,BETA
     &       ,NPLK1,DPE,DME,CPLK,AIB)
            DO 22 J=1,NA0
   22       AII(IU,J,LB)=AIB(J)
           ELSE
CC UPWARD INCREMENT AT TOP OF SUBLAYER (STAMNES INTEGRATION)
            CALL AINTEG(M,LT,NDA,NA0,I,FSOL,AM1U(IU),AM0,W,T1,T1,T2
     &       ,ZEE,QE,QIE,VPE,VME,PT10,PR10,PT1,PR1,ALFA,BETA
     &       ,NPLK1,DPE,DME,CPLK,AIB)
            DO 23 J=1,NA0
   23       AII(IU,J,L)=AIB(J)
          ENDIF
C AT USER DEFINED DEPTH
          DO 24 IT=1,NTAU
          L1=IITAU(IT)
CC  WE DO NOT CALCULATE THE FIELD AT THIS STAGE IF THE USER DEFINED
CC     DEPTH IS EXACTLY ON THE INTERFACES (L1.LT.0).
          IF(L1.EQ.L) THEN
           CALL AINTEG(M,LT,NDA,NA0,I,FSOL,AM1U(IU),AM0,W,UTAU(IT),T1,T2
     &      ,ZEE,QE,QIE,VPE,VME,PT10,PR10,PT1,PR1,ALFA,BETA
     &      ,NPLK1,DPE,DME,CPLK,AIB)
            DO 25 J=1,NA0
   25       AI(IU,J,IT)=AIB(J)
          ENDIF
   24     CONTINUE
   21     CONTINUE
        ENDIF
   13 CONTINUE
      IF(INDA.LE.0 .OR. INDA.EQ.2) RETURN
C--- ADDING INTENSITIES (INDA=1)
      DO 26 IU=1,NA1U
      I=IIAM1(IU)
      IF(AM1U(IU).GT.0.0) THEN
CC DOWNWARD
        DO 27 J=1,NA0
   27   AIB(J)=0
        DO 28 L=1,NLN1
        IF(L.GT.1) THEN
           EX1=-(DPT(L)-DPT(L-1))/AM1(I)
           EX1=EXPFN(EX1)
          DO 29 J=1,NA0
   29     AIB(J)=AIB(J)*EX1+AII(IU,J,L)
        ENDIF
        DO 30 IT=1,NTAU
        LU=IITAU(IT)
        IF(IABS(LU).EQ.L) THEN
          IF(LU.LT.0) THEN
            DO 31 J=1,NA0
   31       AI(IU,J,IT)=AIB(J)
           ELSE
            DTAU=UTAU(IT)-DPT(L)
            EX1=-DTAU/AM1(I)
            EX1=EXPFN(EX1)
            DO 32 J=1,NA0
   32       AI(IU,J,IT)=AI(IU,J,IT)+AIB(J)*EX1
          ENDIF
        ENDIF
   30   CONTINUE
   28   CONTINUE
CC UPWARD ADDING
       ELSE
        DO 33 J=1,NA0
C 93.5.4
        IF(INDG.LT.0) then
          AIB(J)=0
         ELSE
          IF(INDG.EQ.0) THEN
            AIB(J)=RUP(1,J,NLN1)/WMP(1)
           else
C Interpolation of reflected intensities subtracted single scattering
            IF(IMTHD.LE.0) then
              DO 51 K=1,NDA
              K1=NDA-K+1
C 94.5.7 Bug  YY(K1)=RUP(K,J,NLN1)
C Corrected
              YY(K1)=RUP(K,J,NLN1)/WMP(K)
   51         XX(K1)=AMUA(K)
             else
              DO 50 K=1,NDA
              K1=NDA-K+1
C 94.5.7 Bug  YY(K1)=RUP(K,J,NLN1)-SER(K,J,NLN1)
C Corrected
C 97.3.17 Debug By Takashi Nakajima
C              YY(K1)=(RUP(K,J,NLN1)-SER(K,J,NLN1))/WMP(K)
              YY(K1)=(RUP(K,J,NLN1)-SER(K,J,NLN1))/WMP(K)
     &        + ERT(K,J)/WMP(K)
   50         XX(K1)=AMUA(K)
            endif
            CALL CSPL1(KNDM,NDA,XX,YY,A,B,C,D)
            X1=AM1(I)
C 94.5.7 Bug  AIB(J)=CSPLI(X1,NDA,XX,A,B,C,D)/WMP(1)
C Corrected
            AIB(J)=CSPLI(X1,KNDM,NDA,XX,A,B,C,D)
C            write(*,*) (YY(I),i=1,NDA),AIB(J)
          ENDIF
        ENDIF
C 93.5.4 end
   33   CONTINUE
        DO 34 L=NLN1,1,-1
        IF(L.LE.NLN) THEN
           EX1=-(DPT(L+1)-DPT(L))/AM1(I)
           EX1=EXPFN(EX1)
          DO 35 J=1,NA0
   35     AIB(J)=AIB(J)*EX1+AII(IU,J,L)
        ENDIF
        DO 36 IT=1,NTAU
        LU=IITAU(IT)
        IF(LU.LT.0) THEN
          IF(IABS(LU).EQ.L) THEN
            DO 37 J=1,NA0
   37       AI(IU,J,IT)=AIB(J)
          ENDIF
         ELSE
          IF(LU.EQ.L-1) THEN
            DTAU=DPT(L)-UTAU(IT)
            EX1=-DTAU/AM1(I)
            EX1=EXPFN(EX1)
            DO 38 J=1,NA0
   38       AI(IU,J,IT)=AI(IU,J,IT)+AIB(J)*EX1
          ENDIF
        ENDIF
   36   CONTINUE
   34   CONTINUE
      ENDIF
   26 CONTINUE
      RETURN
      END
