      SUBROUTINE GETPH3Z(IUK,IUR,ISPC,WLC,PR,CR1W,CI1W
     & ,NANG,INTVL,IPOL,ANG,SZP,R,V,PH,CEXT,CABS,CG,ERR)
C GET PHASE FUNCTION AND CROSS SECTION FROM TABLE OF INTERPOLATED REFRACTIVE IND
C WITH LOG-REGULAR GRID WAVELENGTHS.
C--- HISTORY
C 93.10.21  CREATED FROM GTCRS2
C 94. 1.13  PH(L,4) -> PH(L,1)
C 95. 3.13  M.Tsukamoto: Eliminate ELDR, Revise CG calculation
C     8.29  Bug in sellection routine of refractive index (Noguchi)
C 96. 7.12  DWL=LOG(WL1/WLMN)/ALOGW*NWL+1 -> DWL=LOG(WL1/WLMN)/ALOGW*(NWL-1)+1
C 00.11.27  Debug in CR1 and CR1 output. Add CR1W and CI1W and output.
C           by T.Y.Nakajima
C 07.08.14  Updated from GETPH3B to take into account large particles
C           typical of precipitation. The built-in Mie Kernel is extended
C           to larger size parameters assuming the geopmetrical optical
C           approximation.
C--- INPUT
C IUK     I     READ UNIT NUMBER OF THE KERNEL FILE.
C IUR     I     READ UNIT NUMBER OF THE REFRACTIVE INDEX TABLE WITH
C                LOG-REGULAR GRID OF WAVELENGTH.
C ISPC    I     SPECIES OF PARTICULATES
C               1: WATER,   2: ICE,   3: DUST-LIKE,    4:  WATER-SOLUBLE
C               5: OCEANIC, 6:  SOOT, 7: VOLCANIC-ASH, 8: 75%H2SO4
C VOLSPC  F     USER DEFINED VOLUME SPECTRAM (DECLARE BY EXTERNAL STATEMENT IN
C               MAIN ROUTINE.
C WLC     R     WAVELENGTH IN CM
C--- OUTPUT
C NANG    I      NBR OF SCATTERING ANGLE
C INTVL   I      NBR OF SIZE PARAMETER
C IPOL    I      NBR OF POLARIZATION COMPONENT
C ANG  R(NANG)   SCATTERING ANGLE IN DEGREE
C SZP  R(INTVL)  SIZE PARAMETER
C R    R(INTVL)  PARTICLE SIZE IN CM
C V    R(INTVL)  DV/DLNR
C PH   R(KNANG,IPOL) VOLUME PHASE FUNCTION
C CEXT   R       EXTINCTION CROSS SECTION
C CABS   R       ABSORPTION CROSS SECTION
C CG     R       GEOMETRICAL CROSS SECTION
C                UNIT IN CM-1
C
C
      include 'VISIRparameters.inc'
c
      SAVE
C      PARAMETER (KNANG =80)
C      PARAMETER (KINTVL=80)
C      PARAMETER (KPOL  =1)
      PARAMETER (KNANG2=KNANG+2)
      CHARACTER ERR*(*)
      DIMENSION ANG(KNANG),SZP(KINTVL),R(KINTVL),V(KINTVL)
     & ,PH(KNANG,KPOL),PR(6,4)
c
c !!! NEW in getph3z.f to extrapolate the Mie Kernel     !!!
c !!! for large size parameters assuming the geometrical !!!
c !!! optical approximation.                             !!!
c
      Real, parameter :: INTVL2 = 10
      Real :: EZRext_GO,EZRabs_GO
      Real, dimension(INTVL2), save :: SZP2 = 
     & (/ 1.e3, 2.e3, 3.e3, 4.e3, 5.e3, 6.e3, 7.e3, 8.e3, 9.e3, 1.e4 /)

C WORKING AREA
      PARAMETER (KSEC=3,KR=20,KI=20)
      PARAMETER (KWL=210,KNV=8,KRF=200)
      PARAMETER (PI=3.141592654)
      CHARACTER CH*1
      DIMENSION WL(KWL),CRW(KWL,KNV),CIW(KWL,KNV)
      DIMENSION CR(KRF),CI(KRF),Q(KINTVL,KPOL,KRF,KNANG2)
     & ,NR(KSEC),NI(KSEC),CRMN(KSEC),CRMX(KSEC),CIMN(KSEC)
     & ,CIMX(KSEC),CRT(KR,KSEC),CIT(KI,KSEC),ICT(KR,KI,KSEC)
      DATA   NR/  3,      9,     18/
      DATA   NI/ 16,     10,      4/
      DATA CRMN/  1.3,    1.0,    1.0/
      DATA CRMX/  1.5,    1.8,    2.7/
      DATA CIMN/  1.0E-9, 1.0E-4, 1.0E-1/
      DATA CIMX/  1.0E-4, 1.0E-1, 1.0E+0/
      DATA INIT/1/
C
      IF(INIT.GT.0) THEN
        INIT=0
C READ REFRACTIVE INDEX TABLE WITH REGULAR WAVELENGTH GRID
        REWIND IUR
        REWIND IUK
        READ(IUR,*) NWL
        NV=0
    4   READ(IUR,*,ERR=1,END=1) NV1
        NV=NV+NV1
        IF(NV.GT.KNV) THEN
          ERR='NV .GT. KNV'
          RETURN
        ENDIF
        READ(IUR,2) CH
        READ(IUR,2) CH
    2   FORMAT(A1)
        DO 3 I=1,NWL
    3   READ(IUR,*) WL(I),(CRW(I,J),CIW(I,J),J=NV-NV1+1,NV)
        GOTO 4
    1   WLMN=WL(1)
        WLMX=WL(NWL)
        ALOGW=LOG(WLMX/WLMN)
C GET KERNEL ANG, SZP, AK, CEXT, CABS
        READ(IUK, *) INTVL, NANG, IPOL
        IF (NANG .GT. KNANG) THEN
          ERR = 'NANG.GT.KNANG'
          RETURN
        END IF
        IF (IPOL .GT. KPOL) THEN
          ERR = 'IPOL.GT.KPOL'
          RETURN
        END IF
        READ(IUK, *) (SZP(I), I = 1, INTVL)
        READ(IUK, *) (ANG(I), I = 1, NANG)
        RSZP2=EXP(LOG(SZP(INTVL)/SZP(1))/(INTVL-1)/2)
CC READ KERNEL
        NRF=0
    7   READ(IUK,2,END=5,ERR=5) CH
        NRF=NRF+1
        IF(NRF.GT.KRF) THEN
          ERR='NRF .GT. KRF'
          RETURN
        ENDIF
        READ(IUK,*) CR(NRF),CI(NRF)
        CI(NRF)=ABS(CI(NRF))
        DO 60 I = 1, INTVL
        DO 60 J = 1, IPOL
   60     READ(IUK, *) (Q(I, J, NRF, K), K = 1, NANG)
        DO 62 K = 1, 2
   62     READ(IUK, *) (Q(I, 1, NRF, K+NANG), I = 1, INTVL)
        GOTO 7
    5   CONTINUE
CC GET REFRACTIVE INDEX TABLES (3 TABLES)
        DO 8 K=1,KSEC
        DCR=(CRMX(K)-CRMN(K))/(NR(K)-1)
        DO 9 I=1,NR(K)
        CR1=CRMN(K)+DCR*(I-1)
        IF(ABS(CR1-1.0).LE.0) CR1=1.01
    9   CRT(I,K)=CR1
        DCI=ALOG(CIMX(K)/CIMN(K))/(NI(K)-1)
        DO 10 I=1,NI(K)
        CI1=DCI*(I-1)
   10   CIT(I,K)=CIMN(K)*EXP(CI1)
    8   CONTINUE
CC FIND GRIDS FOR MKERNEL REFRACTIVE INDICES
        DO 11 K=1,KSEC
        DO 11 I=1,NR(K)
        DO 11 J=1,NI(K)
   11   ICT(I,J,K)=0
        DO 12 L=1,NRF
        CR1=CR(L)
        CI1=CI(L)
        DO 13 K=1,KSEC
        DO 14 I=1,NR(K)
        IF(ABS(CRT(I,K)/CR1-1).LE.0.01) GOTO 15
   14   CONTINUE
        GOTO 13
   15   DO 17 J=1,NI(K)
        IF(ABS(CIT(J,K)/CI1-1).LE.0.01) GOTO 16
   17   CONTINUE
        GOTO 13
   16   ICT(I,J,K)=L
   13   CONTINUE
   12   CONTINUE
        DO 18 K=1,KSEC
        DO 18 I=1,NR(K)
        DO 18 J=1,NI(K)
        IF(ICT(I,J,K).EQ.0) THEN
          ERR='NO MATCHING GRID'
          RETURN
        ENDIF
   18   CONTINUE
C NUMBER OF AVERAGING
        NAV=10
      ENDIF
C GET REFRACTIVE INDEX, ASSUMING LOG-REGULAR GRID
      WL1=WLC*1.0E4
      WVN=2*PI/WLC
C bug fix 96.07   DWL=LOG(WL1/WLMN)/ALOGW*NWL+1
      DWL=LOG(WL1/WLMN)/ALOGW*(NWL-1)+1
      IWL=DWL
      IF(IWL.LE.0  ) IWL=1
      IF(IWL.GE.NWL) IWL=NWL-1
      IWL1=IWL+1
      CR1=CRW(IWL,ISPC)+(CRW(IWL1,ISPC)-CRW(IWL,ISPC))*(DWL-IWL)
      CI1=CIW(IWL,ISPC)+(CIW(IWL1,ISPC)-CIW(IWL,ISPC))*(DWL-IWL)
C2000.11.27 add cr1w and ci1w for ocean reflection.
      cr1w=CRW(IWL,1)+(CRW(IWL1,1)-CRW(IWL,1))*(DWL-IWL)
      ci1w=CIW(IWL,1)+(CIW(IWL1,1)-CIW(IWL,1))*(DWL-IWL)
      DO 23 KP=1,KSEC
      IF(CI1.LE.CIMX(KP)) GOTO 25
   23 CONTINUE
      KP=KSEC
   25 NS=NR(KP)
      NT=NI(KP)
      DO 26 IP=1,NS-1
      IF(CR1.LE.CRT(IP,KP)) GOTO 27
   26 CONTINUE
      IP=NS
   27 DO 28 JP=1,NT-1
      IF(CI1.LE.CIT(JP,KP)) GOTO 29
   28 CONTINUE
C     JP=NT-1  , bug 95.08.29
      JP=NT
   29 IP=MAX(1, IP-1)
C     IJ=MAX(1, IJ-1)  , bug 95.08.29
      JP=MAX(1, JP-1)
      IRF00=ICT(IP  ,JP  ,KP)
      IRF10=ICT(IP+1,JP  ,KP)
      IRF01=ICT(IP  ,JP+1,KP)
      IRF11=ICT(IP+1,JP+1,KP)
C GET INTERPOLATED OPTICAL CONSTANTS
      CG=0
      CEXT=0
      CABS=0
      DO 20 L=1,NANG
C  20 PH(L,4)=0
   20 PH(L,1)=0
CC
      DO 30 IR=1,INTVL
      R1=SZP(IR)/WVN/RSZP2
      R2=SZP(IR)/WVN*RSZP2
      DR=(R2-R1)/NAV
CC* 95. 3.13 M.Tsukamoto      ELDR=LOG(R2/R1)
      R3=R1-DR/2
      V1=0.0
      DO 34 J=1,NAV
      R3=R3+DR
CC* 95. 3.13 M.Tsukamoto
CC*      V1=V1+VOLSPC(R3)
CC*   34 CG=CG+V1*3.0/4.0/R3*LOG(R3/(R3-DR))
CC*      R(IR)=SQRT(R1*R2)
CC*   30 V(IR)=V1/REAL(NAV)
      PR(1,1) = R3
      V2=VLSPC3(PR)
C      V2=VOLSPC(R3)
      CG=CG+V2*3.0/4.0/R3*LOG(R3/(R3-DR))
   34 V1=V1+V2
      R(IR)=SQRT(R1*R2)
   30 V(IR)=V1/REAL(NAV)
C INTERPOLATION
      DO 31 IR=1,INTVL
      DO 31 L=1,NANG+2
      PZ00 = LOG(Q(IR, 1, IRF00, L))
      PZ10 = LOG(Q(IR, 1, IRF10, L))
      PZ01 = LOG(Q(IR, 1, IRF01, L))
      PZ11 = LOG(Q(IR, 1, IRF11, L))
      Z1=PZ00+(CR1-CRT(IP,KP))/(CRT(IP+1,KP)-CRT(IP,KP))*(PZ10-PZ00)
      Z2=PZ01+(CR1-CRT(IP,KP))/(CRT(IP+1,KP)-CRT(IP,KP))*(PZ11-PZ01)
      Z=Z1+LOG(CI1/CIT(JP,KP))/LOG(CIT(JP+1,KP)/CIT(JP,KP))*(Z2-Z1)
c for use by the extrapolation later
      If ( IR==INTVL .and. L==NANG+1 ) EZRext_GO = exp(Z)*R(IR)
      If ( IR==INTVL .and. L==NANG+2 ) EZRabs_GO = exp(Z)*R(IR)
c
      Z=EXP(Z)*V(IR)*WVN
      IF(L.LE.NANG) THEN
C      PH(L,4)=PH(L,4)+Z
       PH(L,1)=PH(L,1)+Z
       ELSE IF(L.EQ.NANG+1) THEN
        CEXT=CEXT+Z
       ELSE IF(L.EQ.NANG+2) THEN
        CABS=CABS+Z
       END IF
   31 CONTINUE
c
c !!! Extrapolation for large size parameters (added in geph3z) !!!
c ->->->
c
      RSZP2=EXP(LOG(SZP2(INTVL2)/SZP2(1))/(INTVL2-1)/2)
      DO IR=1,INTVL2
      R1=SZP2(IR)/WVN/RSZP2
      R2=SZP2(IR)/WVN*RSZP2
      DR=(R2-R1)/NAV
      R3=R1-DR/2
      V1=0.0
      DO J=1,NAV
      R3=R3+DR
      PR(1,1) = R3
      V2=VLSPC3(PR)
      CG=CG+V2*3.0/4.0/R3*LOG(R3/(R3-DR))
      V1=V1+V2
      End do
      R(IR)=SQRT(R1*R2)
      V(IR)=V1/REAL(NAV)
      End do
C INTERPOLATION
      DO IR=1,INTVL2
      Z=EZRext_GO*V(IR)*WVN/R(IR)
      CEXT=CEXT+Z
      Z=EZRabs_GO*V(IR)*WVN/R(IR)
      CABS=CABS+Z
      End do
C <-<-<-
      ERR=' '
      RETURN
      END
