      SUBROUTINE SCALD(PP,TT,W,IBMOL,DENSTY)
C EFFECTIVE GAS ABSORPTION AMOUNT
C--- HISTORY
C 92. 6.27 CREATED
C--- INPUT
C PP       R        PRESSURE IN MB
C TT       R        TEMPERATURE IN K
C W        R(28)    GAS AMOUNT IN PPMV (FIRST 11 ARE MEANINGFUL)
C---
C IBMOL    I(53)    BAND NUMBER TO MOLECULAR ID
C                   ABSORBER AMOUNT (PPMV) FOR ID: 1-11 SHOULD BE GIVEN.
C                   ABSORBER AMOUNT = 1    FOR ID: 22   SHOULD BE GIVEN.
C                   OTHERWISE ZERO.
C DENSTY   R(53)    EFFECTIVE GAS AMOUNT FOR THE BANDS DEFINED BY IBMOL
C                   UNIT IS DIFFERENT EACH BY EACH
C MOLECULAR CODE
C  1: H2O,  2: CO2,   3: O3,   4: N2O,  5: CO,  6: CH4,  7: O2
C  8: NO,   9: SO2,  10: NO2, 11: NH3
C     K
C     1-41    MOLECULAR LINE ABSORPTION
C     42-44   WATER VAPOR CONTINUUM ABSORPTION
C     45      N2 CONTINUUM
C     46      ULTRAVIOLET OZONE
C     47      HNO3 (NITRIC ACID)
C     48-50   OXYGEN
C     51      OXYGEN HERZBERG CONTINUUM
C     52-53   ULTRAVIOLET OZONE
C
      SAVE
      DIMENSION  W(28),IBMOL(53),DENSTY(53)
      DIMENSION IBMOL1(53)
C
      DATA PZERO /1013.25/,TZERO/273.15/
C  XLOSCH: LOSCHMIDT'S NUMBER (MOLECULES CM-2,KM-1)
C AXLOSCH:                    (MOLECULES CM-3)
      DATA XLOSCH/2.6868E24/,AXLOSCH/2.6868E19/
C CON CONVERTS WATER VAPOR FROM GM M-3 TO MOLECULES CM-2 KM-1
      DATA CON/3.3429E21/
C CONJOE=(1/XLOSCH)*1.E5*1.E-6 WITH
C        1.E5 ARISING FROM CM TO KM CONVERSION AND
C        1.E-6  "       "  PPMV
C CONJOE IS USED TO CHANGE PPMV TO ATM CM/KM
      DATA CONJOE/3.7194E-21/
C H20 CONTINUUM IS STORED AT 296 K RHZERO IS AIR DENSITY AT 296 K
C IN UNITS OF LOSCHMIDT'S. RHZERO=(273.15/296.0)
      DATA RHZERO/0.922804/
      DATA IBMOL1/ 1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
     &             1,  1,  1,  1,  3,  3,  3,  3,  3,  2,
     &             2,  2,  2,  2,  2,  2,  2,  5,  5,  6,
     &             4,  4,  4,  7,  7, 11, 11,  8, 10,  9,
     &             9,  1,  1,  1, 22,  3, 12,  7,  7,  7,
     &             7,  3,  3/
      DATA INIT/1/
      IF(INIT.GT.0) THEN
        INIT=0
        DO 1 K=1,53
    1   IBMOL(K)=IBMOL1(K)
      ENDIF
      PSS=PP/PZERO
      TSS=TZERO/TT
C AIR MOLECULES/CM3
      WAIR = AXLOSCH * PSS * TSS
C AAPV TO G/M3 (2.989E-23 = 18 G/AVOGADRO NUMBER)
      WH1    = 2.989E-23 *WAIR*W(1)
      WTEMP=WH1
C     UV OZONE (PPMV TO ATM CM/KM)
      DENSTY(46)= CONJOE   *WAIR    *W(3)
C     N2 CONTINUUM
      F1=PSS * TSS
      F2=PSS * SQRT(TSS)
      DENSTY(45)=0.781*F1*F2
C     SELF BROADENED WATER
      RHOAIR = F1
      RHOH2O = CON *WTEMP/XLOSCH
      RHOFRN = RHOAIR - RHOH2O
C PPMV TO MOLECULES/(CM2 KM)
      DENSTY(42)= XLOSCH*RHOH2O**2/RHZERO
C     FOREIGN BROADENED
      DENSTY(44)= XLOSCH*RHOH2O*RHOFRN/RHZERO
      DENSTY(43)=0.
C****************************************************
C   HNO3 IS SUBTRACTED FROM LOWTRAN
      DENSTY(47)=0.
C****************************************************
C      O2 IN ATM CM/KM
C      O2 TEMP DEP
C
      DT = TT  - 220.
      WO2D       = CONJOE   *WAIR    *W(7)  * PSS
C
C     DT CAN BE NEGIVATIVE
C     EFFECTIVE DT CALCULATED IN TRANS
C
      DENSTY(48)  = WO2D * TT
      DENSTY(49)  = WO2D * DT * DT
      DENSTY(50)  = WO2D
C
C  --- FOR H2O  G/M3
      CONH2O=WH1  *.1
      DENSTY(1)=CONH2O*PSS**0.9810*TSS**( 0.3324)
      DENSTY(2)=CONH2O*PSS**1.1406*TSS**(-2.6343)
      DENSTY(3)=CONH2O*PSS**0.9834*TSS**(-2.5294)
      DENSTY(4)=CONH2O*PSS**1.0443*TSS**(-2.4359)
      DENSTY(5)=CONH2O*PSS**0.9681*TSS**(-1.9537)
      DENSTY(6)=CONH2O*PSS**0.9555*TSS**(-1.5378)
      DENSTY(7)=CONH2O*PSS**0.9362*TSS**(-1.6338)
      DENSTY(8)=CONH2O*PSS**0.9233*TSS**(-0.9398)
      DENSTY(9)=CONH2O*PSS**0.8658*TSS**(-0.1034)
      DENSTY(10)=CONH2O*PSS**0.8874*TSS**(-0.2576)
      DENSTY(11)=CONH2O*PSS**0.7982*TSS**( 0.0588)
      DENSTY(12)=CONH2O*PSS**0.8088*TSS**( 0.2816)
      DENSTY(13)=CONH2O*PSS**0.6642*TSS**( 0.2764)
      DENSTY(14)=CONH2O*PSS**0.6656*TSS**( 0.5061)
C  --- FOR O3
      CONO3 = CONJOE   *WAIR    *W(3)
      DENSTY(15)=CONO3 *PSS**0.4200*TSS**( 1.3909)
      DENSTY(16)=CONO3 *PSS**0.4221*TSS**( 0.7678)
      DENSTY(17)=CONO3 *PSS**0.3739*TSS**( 0.1225)
      DENSTY(18)=CONO3 *PSS**0.1770*TSS**( 0.9827)
      DENSTY(19)=CONO3 *PSS**0.3921*TSS**( 0.1942)
C  --- FOR CO2
      CONCO2= CONJOE   *WAIR  *W(2)
      DENSTY(20)=CONCO2*PSS**0.6705*TSS**(-2.2560)
      DENSTY(21)=CONCO2*PSS**0.7038*TSS**(-5.0768)
      DENSTY(22)=CONCO2*PSS**0.7258*TSS**(-1.6740)
      DENSTY(23)=CONCO2*PSS**0.6982*TSS**(-1.8107)
      DENSTY(24)=CONCO2*PSS**0.8867*TSS**(-0.5327)
      DENSTY(25)=CONCO2*PSS**0.7883*TSS**(-1.3244)
      DENSTY(26)=CONCO2*PSS**0.6899*TSS**(-0.8152)
      DENSTY(27)=CONCO2*PSS**0.6035*TSS**( 0.6026)
C  --- FOR CO
      CONCO = CONJOE   *WAIR  *W(5)
      DENSTY(28)=CONCO *PSS**0.7589*TSS**( 0.6911)
      DENSTY(29)=CONCO *PSS**0.9267*TSS**( 0.1716)
C  --- FOR CH4
      CONCH4= CONJOE   *WAIR  *W(6)
      DENSTY(30)=CONCH4*PSS**0.7139*TSS**(-0.4185)
C  --- FOR N2O
      CONN2O= CONJOE   *WAIR  *W(4)
      DENSTY(31)=CONN2O*PSS**0.3783*TSS**( 0.9399)
      DENSTY(32)=CONN2O*PSS**0.7203*TSS**(-0.1836)
      DENSTY(33)=CONN2O*PSS**0.7764*TSS**( 1.1931)
C  --- FOR O2
      CONO2 = CONJOE   *WAIR  *W(7)
      DENSTY(34)=CONO2 *PSS**1.1879*TSS**( 2.9738)
      DENSTY(35)=CONO2 *PSS**0.9353*TSS**( 0.1936)
C  --- FOR NH3
      CONNH3= CONJOE   *WAIR  *W(11)
      DENSTY(36)=CONNH3*PSS**0.8023*TSS**(-0.9111)
      DENSTY(37)=CONNH3*PSS**0.6968*TSS**( 0.3377)
C  --- FOR NO
      CONNO = CONJOE   *WAIR  *W(8)
      DENSTY(38)=CONNO *PSS**0.5265*TSS**(-0.4702)
C  --- FOR NO2
      CONNO2= CONJOE   *WAIR  *W(10)
      DENSTY(39)=CONNO2*PSS**0.3956*TSS**(-0.0545)
C  --- FOR SO2
      CONSO2= CONJOE   *WAIR  *W(9)
      DENSTY(40)=CONSO2*PSS**0.2943*TSS**( 1.2316)
      DENSTY(41)=CONSO2*PSS**0.2135*TSS**( 0.0733)
C***********************************************************************
C   HERZBERG CONTINUUM PRESSURE DEPENDENCE CALCULATION, SHARDANAND 1977
C      AND   YOSHINO ET AL 1988
C
C     OXYGEN
C
      DENSTY(51)=(1.+.83*F1)*CONO2
C ADDED FROM -GEO-
      DENSTY(52)=DENSTY(46)*0.269*(TT-273.15)
      DENSTY(53)=DENSTY(46)*0.269*(TT-273.15)**2
      WTEM = (296.0-TT)/(296.0-260.0)
      IF(WTEM.LT.0.0) WTEM = 0.
      IF(WTEM.GT.1.0) WTEM = 1.0
      DENSTY(43) =WTEM*DENSTY(42)
      RETURN
      END
